@@ -93,6 +93,8 @@ void GR_Win32Graphics::_constructorCommonCode(HDC hdc)
 {
 	UT_ASSERT(hdc);
 
+	m_iDCFontAllocNo = 0;
+	m_iPrintDCFontAllocNo = 0;
 	m_hdc = hdc;
 	m_printHDC = NULL;
 	m_nPrintLogPixelsY = 0;
@@ -475,6 +477,7 @@ void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
 	int iLength = iLengthOrig;
 	HFONT hFont = m_pFont->getDisplayFont(this);
 	SelectObject(m_hdc, hFont);
+	m_iDCFontAllocNo = m_pFont->getAllocNumber();
 	SetTextAlign(m_hdc, TA_LEFT | TA_TOP);
 	SetBkMode(m_hdc, TRANSPARENT);		// TODO: remember and reset?
 
@@ -650,7 +653,7 @@ UT_uint32 GR_Win32Graphics::getFontHeight()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
 
-	return (UT_uint32)((m_pFont->getHeight(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getHeight(m_hdc, m_printHDC));
 }
 
 UT_uint32 GR_Win32Graphics::getFontAscent(GR_Font* fnt)
@@ -665,7 +668,7 @@ UT_uint32 GR_Win32Graphics::getFontAscent(GR_Font* fnt)
 UT_uint32 GR_Win32Graphics::getFontAscent()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
-	return (UT_uint32)((m_pFont->getAscent(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getAscent(m_hdc, m_printHDC));
 }
 
 UT_uint32 GR_Win32Graphics::getFontDescent(GR_Font* fnt)
@@ -680,7 +683,7 @@ UT_uint32 GR_Win32Graphics::getFontDescent(GR_Font* fnt)
 UT_uint32 GR_Win32Graphics::getFontDescent()
 {
 	UT_return_val_if_fail( m_pFont, 0 );
-	return (UT_uint32)((m_pFont->getDescent(m_hdc, m_printHDC)) * getResolution() / getDeviceResolution());
+	return (UT_uint32)(m_pFont->getDescent(m_hdc, m_printHDC));
 }
 
 void GR_Win32Graphics::getCoverage(UT_NumberVector& coverage)
@@ -931,8 +934,10 @@ bool GR_Win32Graphics::startPrint(void)
 bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 									bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight)
 {
-	iWidth = (UT_sint32)((double)_tduR(iWidth) * m_fXYRatio);
-	iHeight = _tduR(iHeight);
+	// need these in 10th of milimiters
+	iWidth = (int)((double)iWidth * m_fXYRatio * 254.0 / (double)getResolution() + 0.5);
+	iHeight = iHeight * 254 / getResolution();
+
 	if (m_bStartPage)
 	{
 		EndPage(m_hdc);
@@ -942,8 +947,12 @@ bool GR_Win32Graphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 	if (m_hDevMode)
 	{
 		DEVMODE *pDevMode = (DEVMODE*) GlobalLock(m_hDevMode);
-		pDevMode->dmFields = DM_ORIENTATION;
+		pDevMode->dmFields = DM_ORIENTATION | DM_PAPERLENGTH | DM_PAPERWIDTH;
 		pDevMode->dmOrientation = (bPortrait) ? DMORIENT_PORTRAIT : DMORIENT_LANDSCAPE;
+		pDevMode->dmPaperSize = 0;
+		pDevMode->dmPaperLength = iHeight;
+		pDevMode->dmPaperWidth = iWidth;
+		
 		GlobalUnlock(m_hDevMode);
 
 		// call DocumentProperties() on the DEVMODE to ensure changes propagate down into
@@ -1187,6 +1196,7 @@ void GR_Win32Graphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDes
 	
 	if(!iRes)
 	{
+		SetStretchBltMode(m_hdc, COLORONCOLOR);
 		iRes = StretchDIBits(m_hdc,
 							 xDest, yDest,
 							 pImg->getDisplayWidth(), pImg->getDisplayHeight(),
@@ -1493,10 +1503,12 @@ GR_Win32Font::GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 	
 	if(hdc != printHDC)
 	{
-		int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);
 		int nPrintLogPixelsY = GetDeviceCaps(printHDC, LOGPIXELSY);
 
-		lf.lfHeight = MulDiv(lf.lfHeight, nPrintLogPixelsY, nLogPixelsY);
+		// use the point size rather than the screen pixel size to minimise rounding error
+		// (lfHeight already carries a rounding error from conversion pts -> screen pixels)
+		// lf.lfHeight = MulDiv(lf.lfHeight, nPrintLogPixelsY, nLogPixelsY);
+		lf.lfHeight =  (int)(-fPoints * (double)nPrintLogPixelsY / 72.00 + 0.5);
 		printFont = CreateFontIndirect(&lf);
 		
 		if(!printFont)
@@ -1539,7 +1551,7 @@ GR_Win32Font::GR_Win32Font(LOGFONT & lf, double fPoints, HDC hdc, HDC printHDC)
 			GetTextFace(printHDC, 1000, lpFaceName );
 
 			_updateFontYMetrics(hdc, printHDC);
-			
+
 			UT_String_sprintf(m_hashKey, "%s-%d-%d-%d-%d-%d",
 							  lpFaceName,
 							  m_tm.tmHeight, m_tm.tmWeight, m_tm.tmItalic, m_tm.tmUnderlined,
@@ -1644,13 +1656,18 @@ void GR_Win32Font::_updateFontYMetrics(HDC hdc, HDC printHDC)
 	{
 		GetTextMetrics(printHDC,&m_tm);
 
-		// now we have to scale the metrics down from the printHDC to primary dc
+		// now we have to scale the metrics down from the printHDC to our layout units
+		// 
+		// (we used to scale the metrics down to the device resolution, but that since we
+		// then scale it back up to layout units all the time, this caused us huge
+		// rounding error) Tomas, 16/9/05
+		// 
 		// NB: we do not want to obtain the metrics directly for the primary dc
 		// Windows is designed to get slightly bigger font for the screen than the
 		// user asks for (MS say to improve readibility), and this causes us
 		// non-WYSIWYG behaviour (basically our characters are wider on screen than on
 		// the printer and our lines are further apart)
-		int nLogPixelsY      = GetDeviceCaps(hdc, LOGPIXELSY);
+		int nLogPixelsY      = GR_Graphics::getResolution();
 		int nPrintLogPixelsY = GetDeviceCaps(printHDC, LOGPIXELSY);
 
 		if(nLogPixelsY != nPrintLogPixelsY)
@@ -1748,7 +1765,11 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 
 	// scale the pixel size to the printer resolution and get the font
 	if(!m_bGUIFont)
-		pixels = MulDiv(pixels, nPrintLogPixelsY, 72);
+	{
+		// use point size to reduce rounding errors
+		// pixels = MulDiv(pixels, nPrintLogPixelsY, 72);
+		pixels = (int)(m_fPointSize * (double)nPrintLogPixelsY / 72.0 + 0.5);
+	}
 	
 	HFONT pFont = getFontFromCache(pixels, false, 100);
 	if (!pFont)
@@ -1760,7 +1781,14 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	// select our font into the printer DC
 	HGDIOBJ hRet = SelectObject(printDC, pFont);
 	UT_return_val_if_fail( hRet != (void*)GDI_ERROR, false);
-		
+
+	// remember which font we selected into the dc
+	if(printDC == pWin32Gr->getPrintDC())
+		pWin32Gr->setPrintDCFontAllocNo(getAllocNumber());
+
+	if(printDC == pWin32Gr->getPrimaryDC())
+		pWin32Gr->setDCFontAllocNo(getAllocNumber());
+	
 	if (printDC != m_hdc)
 	{
 		// invalidate cached info when we change hdc's.
@@ -1778,11 +1806,36 @@ bool GR_Win32Font::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG)
 	}
 	else
 	{
-		// TODO: an UT_UCS4Char -> ANSI conversion here, then call GetGlyphOutlineA() ...
-		// Actually, this probably is not much use, since this function is currently only
-		// used by the math plugin, and for that to work, we do need to work with unicode,
-		// not ANSI
-		UT_return_val_if_fail( UT_NOT_IMPLEMENTED, false );
+		// GetGlyphOutlineA() ... using the GGO_GLYPH_INDEX = 0x0080 to get the ttf glyph 
+		// for the unicode character set in ANSI
+
+		// first of all, we need to translate the ucs4 value to the index; we use
+		// GetCharacterPlacementW() for this. Event though the docs state that this
+		// function is only available on win9x via unicows, this is in fact incorrect
+		// (whether the 9x version of the function is fully functional I do not know, but
+		// it does glyph index lookup).
+		UINT iIndx;
+		GCP_RESULTSW gcpResult = {0};
+		gcpResult.lStructSize = sizeof(GCP_RESULTS);
+		
+		// w32api changed lpGlyphs from UINT * to LPWSTR to match MS PSDK in w32api v2.4
+#if defined(__MINGW32__) && (__W32API_MAJOR_VERSION == 2 && __W32API_MINOR_VERSION < 4)
+		gcpResult.lpGlyphs = (UINT *) &iIndx;	// Character glyphs
+#else			
+		gcpResult.lpGlyphs = (LPWSTR) &iIndx;	// Character glyphs
+#endif			
+		gcpResult.nGlyphs = 1;  // Array size
+
+		if(GetCharacterPlacementW(printDC, (LPCWSTR)&g, 1, 0, &gcpResult, 0))
+		{
+			iRet = GetGlyphOutlineA(printDC, iIndx, 0x0080 | GGO_METRICS, &gm, 0, NULL, &m);
+		}
+		else
+		{
+			// OK, the GetCharacterPlacementW function probably not present, we just do
+			// our best with the ANSI stuff
+			iRet = GetGlyphOutlineA(printDC, g, GGO_METRICS, &gm, 0, NULL, &m);
+		}
 	}
 	
 	if(GDI_ERROR == iRet)
