@@ -26,18 +26,22 @@
 typedef struct tagBITMAPINFO BITMAPINFO;
 
 
-class GR_Win32Image : public GR_RasterImage
+class ABI_EXPORT GR_Win32Image : public GR_RasterImage
 {
 public:
 	GR_Win32Image(const char* szName);
-	~GR_Win32Image();
+	virtual ~GR_Win32Image();
 
 	virtual bool		convertToBuffer(UT_ByteBuf** ppBB) const;
 	virtual bool		convertFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 	
-	void				setDIB(BITMAPINFO *pDIB) { m_pDIB = pDIB; }
+	void				setDIB(BITMAPINFO *pDIB) { m_pDIB = pDIB; if (m_pDIB) setDisplaySize(m_pDIB->bmiHeader.biWidth,m_pDIB->bmiHeader.biHeight); }
 	inline BITMAPINFO*	getDIB(void) const { return m_pDIB; }
 
+	virtual GR_Image *  createImageSegment(GR_Graphics * pG, const UT_Rect & rec);
+	virtual bool hasAlpha (void) const;
+	virtual bool isTransparentAt(UT_sint32 x, UT_sint32 y);
+
 protected:
 	BITMAPINFO*		m_pDIB;
 };
