@@ -20,12 +20,18 @@
 #ifndef UT_BASE64_H
 #define UT_BASE64_H
 
-#include "ut_assert.h"
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
-#include "ut_bytebuf.h"
+#endif
 
-bool ABI_EXPORT UT_Base64Encode(UT_ByteBuf * pDest, const UT_ByteBuf * pSrc);
-bool ABI_EXPORT UT_Base64Decode(UT_ByteBuf * pDest, const UT_ByteBuf * pSrc);
+class UT_ByteBuf;
+
+ABI_EXPORT bool UT_Base64Encode(UT_ByteBuf * pDest, const UT_ByteBuf * pSrc);
+ABI_EXPORT bool UT_Base64Decode(UT_ByteBuf * pDest, const UT_ByteBuf * pSrc);
 
 ABI_EXPORT bool UT_UTF8_Base64Encode(char *& b64ptr, size_t & b64len, const char *& binptr, size_t & binlen);
 ABI_EXPORT bool UT_UTF8_Base64Decode(char *& binptr, size_t & binlen, const char *& b64ptr, size_t & b64len);
