@@ -1,5 +1,5 @@
 /* AbiWord
- * Copyright (C) 2002-2003 Hubert Figuiere
+ * Copyright (C) 2002-2005 Hubert Figuiere
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,13 +24,17 @@
 
 #import <Cocoa/Cocoa.h>
 #include "ut_string_class.h"
+#include "ut_vector.h"
+
 #include "xap_Strings.h"
 
 class XAP_Dialog;
 
+void SetNSControlLabel (id control, const UT_UTF8String &label);
 void LocalizeControl (id control, const XAP_StringSet * pSS, XAP_String_Id stringId);
 NSString* LocalizedString (const XAP_StringSet * pSS, XAP_String_Id stringId);
 void AppendLocalizedMenuItem (NSPopUpButton* menu, const XAP_StringSet * pSS, XAP_String_Id stringId, int tag);
+void FillPopupWithCStrVector(NSPopUpButton* menu, const UT_GenericVector<const char*>& vec);
 void _convertLabelToMac (char * buf, size_t bufSize, const UT_UTF8String & label);
 
 @protocol XAP_CocoaDialogProtocol
