@@ -16,7 +16,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
  * 02111-1307, USA.
  */
+
 #undef GTK_DISABLE_DEPRECATED
+
 #ifdef USE_GUCHARMAP
 #include <gucharmap/gucharmap.h>
 #endif
@@ -76,7 +78,6 @@ XAP_UnixDialog_Insert_Symbol::XAP_UnixDialog_Insert_Symbol(XAP_DialogFactory * p
 	m_windowMain = NULL;
 	m_SymbolMap = 	NULL;
 	m_InsertS_Font_list = NULL;
-	m_Insert_Symbol_no_fonts = 0;
 	m_fontlist.clear();
 
 #ifndef USE_GUCHARMAP
@@ -122,8 +123,7 @@ void XAP_UnixDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
 	GtkWidget * mainWindow = _constructWindow();
 	UT_return_if_fail(mainWindow);
 
-	abiSetupModelessDialog(GTK_DIALOG(mainWindow),
-						   pFrame, this, BUTTON_CLOSE);
+	abiSetupModelessDialog(GTK_DIALOG(mainWindow), pFrame, this, BUTTON_INSERT);
 
 #ifndef USE_GUCHARMAP
 
@@ -136,22 +136,27 @@ void XAP_UnixDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
 
 	// make a new Unix GC
 	DELETEP (m_unixGraphics);
-
-	m_unixGraphics = new GR_UnixGraphics(m_SymbolMap->window, unixapp->getFontManager(), m_pApp);
-
+	
+	{
+		//m_unixGraphics = new GR_UnixGraphics(m_SymbolMap->window, unixapp->getFontManager(), m_pApp);
+		GR_UnixAllocInfo ai(m_SymbolMap->window, unixapp->getFontManager(), m_pApp);
+		m_unixGraphics = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+	}
 	// let the widget materialize
 	_createSymbolFromGC(m_unixGraphics,
 						static_cast<UT_uint32>(m_SymbolMap->allocation.width),
 						static_cast<UT_uint32>(m_SymbolMap->allocation.height));
-
+	
 	// *** Re use the code to draw into the selected symbol area.
 	UT_ASSERT(m_areaCurrentSym && m_areaCurrentSym->window);
-
+	
 	// make a new Unix GC
 	DELETEP (m_unixarea);
-
-	m_unixarea = new GR_UnixGraphics(m_areaCurrentSym->window, unixapp->getFontManager(), m_pApp);
-		
+    {
+		//m_unixarea = new GR_UnixGraphics(m_areaCurrentSym->window, unixapp->getFontManager(), m_pApp);
+		GR_UnixAllocInfo ai(m_areaCurrentSym->window, unixapp->getFontManager(), m_pApp);
+		m_unixarea = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+	}
 	// let the widget materialize
 	_createSymbolareaFromGC(m_unixarea,
 							static_cast<UT_uint32>(m_areaCurrentSym->allocation.width),
@@ -229,9 +234,9 @@ void XAP_UnixDialog_Insert_Symbol::event_WindowDelete(void)
 		xap_Unix_Prev_Font = iDrawSymbol->getSelectedFont();
 	g_list_free(m_InsertS_Font_list);
 	
-	for(UT_uint32 i = 0; i < m_Insert_Symbol_no_fonts; i++) 
+	for(UT_uint32 i = 0; i < m_fontlist.getItemCount(); i++) 
 	{
-		free(static_cast<gchar *>(m_fontlist.getNthItem(i)));
+		free(m_fontlist.getNthItem(i));
 	}
 	m_fontlist.clear();
 	modeless_cleanup();
@@ -472,8 +477,9 @@ void XAP_UnixDialog_Insert_Symbol::CurrentSymbol_clicked(GdkEvent *event)
 void XAP_UnixDialog_Insert_Symbol::destroy(void)
 {
 	g_list_free( m_InsertS_Font_list);
-	for(UT_uint32 i = 0; i < m_Insert_Symbol_no_fonts; i++) 
-		 free( static_cast<gchar *>(m_fontlist.getNthItem(i)));
+	for(UT_uint32 i = 0; i < m_fontlist.getItemCount(); i++) 
+		 free(m_fontlist.getNthItem(i));
+	m_fontlist.clear();
 
 	modeless_cleanup();
 	
@@ -486,19 +492,20 @@ void XAP_UnixDialog_Insert_Symbol::destroy(void)
 
 GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 {
-	GtkWidget * vboxInsertS;
+	const XAP_StringSet * pSS = m_pApp->getStringSet();
+	GtkWidget *tmp;
 
 	ConstructWindowName();
 
-	m_windowMain = abiDialogNew ( "insert symbol dialog", TRUE, m_WindowName ) ;
+	m_windowMain = abiDialogNew ("insert symbol dialog", TRUE, m_WindowName);
 
 	// Now put in a Vbox to hold our 3 widgets (Font Selector, Symbol Table
 	// and OK -Selected Symbol- Cancel
-	vboxInsertS = GTK_DIALOG(m_windowMain)->vbox ;
+	tmp = GTK_DIALOG(m_windowMain)->vbox ;
 
 	GtkWidget * hbox = gtk_hbox_new (FALSE, 4);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start(GTK_BOX(vboxInsertS), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(tmp), hbox, FALSE, FALSE, 0);
 
 	// Finally construct the combo box
 	m_fontcombo = _createComboboxWithFonts ();
@@ -510,24 +517,25 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 	// Now the Symbol Map. 
 	// TODO: 32 * x (19) = 608, 7 * y (21) = 147  FIXME!
 	m_SymbolMap = _previewNew (608, 147);
-	gtk_box_pack_start(GTK_BOX(vboxInsertS), m_SymbolMap, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(tmp), m_SymbolMap, FALSE, FALSE, 0);
 	
 	m_areaCurrentSym = _previewNew (60, 45);
-	gtk_box_pack_start(GTK_BOX(hbox),
-					   m_areaCurrentSym, TRUE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), m_areaCurrentSym, TRUE, FALSE, 0);
 #else
-
-	m_SymbolMap = gucharmap_charmap_new ();
+	m_SymbolMap = gucharmap_charmap_new (
+		GUCHARMAP_CHAPTERS(gucharmap_block_chapters_new ()));
 	gtk_widget_show (m_SymbolMap);
-	gtk_box_pack_start(GTK_BOX(vboxInsertS), m_SymbolMap, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (tmp), m_SymbolMap, TRUE, TRUE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (m_SymbolMap), 5);
 
 	gtk_widget_set_size_request (m_windowMain, 700, 300);
 #endif
 
 	gtk_widget_show_all (hbox);
 
-	abiAddStockButton (GTK_DIALOG(m_windowMain), GTK_STOCK_ADD, BUTTON_INSERT) ;
-	abiAddStockButton (GTK_DIALOG(m_windowMain), GTK_STOCK_CLOSE, BUTTON_CLOSE) ;
+	abiAddStockButton (GTK_DIALOG(m_windowMain), GTK_STOCK_CLOSE, BUTTON_CLOSE);
+	tmp = abiAddButton (GTK_DIALOG(m_windowMain), "&Insert" /* not used */, BUTTON_INSERT);
+	localizeButtonUnderline (tmp, pSS, XAP_STRING_ID_DLG_Insert);
 	
 	_connectSignals ();
 
@@ -544,7 +552,7 @@ GtkWidget * XAP_UnixDialog_Insert_Symbol::_constructWindow(void)
 GList *XAP_UnixDialog_Insert_Symbol::_getGlistFonts (void)
 {	  
 	XAP_UnixApp * unixapp = static_cast<XAP_UnixApp *> (m_pApp);
-	UT_Vector * list = unixapp->getFontManager()->getAllFonts();
+	UT_GenericVector<XAP_UnixFont*>* list = unixapp->getFontManager()->getAllFonts();
 	UT_uint32 count = list->size();
 
 	GList *glFonts = NULL;
@@ -553,20 +561,18 @@ GList *XAP_UnixDialog_Insert_Symbol::_getGlistFonts (void)
 
 	for (UT_uint32 i = 0; i < count; i++)
 	{
-		XAP_UnixFont * pFont = static_cast<XAP_UnixFont *>(list->getNthItem(i));
+		const XAP_UnixFont * pFont = list->getNthItem(i);
 		const gchar * lgn  = static_cast<const gchar *>(pFont->getName());
 		
 		if((strstr(currentfont.c_str(),lgn)==NULL) || (currentfont.size() !=strlen(lgn)) )
 		{
 			currentfont = lgn;
-			m_fontlist.addItem(static_cast<void *>( UT_strdup(currentfont.c_str())));
-			glFonts = g_list_prepend(glFonts, static_cast<gchar *>(m_fontlist.getNthItem(j)));
+			m_fontlist.addItem(static_cast<gchar*>(UT_strdup(currentfont.c_str())));
+			glFonts = g_list_prepend(glFonts, m_fontlist.getNthItem(j));
 			j++;
 		}
 	}	
 
-	m_Insert_Symbol_no_fonts = j;
-
 	DELETEP(list);
 	return g_list_reverse(glFonts);
 }
@@ -577,14 +583,10 @@ GtkWidget *XAP_UnixDialog_Insert_Symbol::_createComboboxWithFonts (void)
 	gtk_widget_show(fontcombo);
 
 	m_InsertS_Font_list = _getGlistFonts ();
- 
-	gtk_widget_set_size_request(fontcombo, 200, 25);
-	gtk_combo_set_value_in_list(GTK_COMBO(fontcombo), TRUE, TRUE);
-	gtk_combo_set_use_arrows(GTK_COMBO(fontcombo), FALSE);
 	gtk_combo_set_popdown_strings(GTK_COMBO(fontcombo), m_InsertS_Font_list);
 
 	// Turn off keyboard entry in the font selection box
-	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(fontcombo)->entry),FALSE);
+	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(fontcombo)->entry), FALSE);
 
 	return fontcombo;
 }
