@@ -25,6 +25,7 @@
 #include "xap_Dlg_FontChooser.h"
 #include "ut_string_class.h"
 #include "gr_Graphics.h"
+#include "gr_Painter.h"
 
 /*****************************************************************/
 
@@ -504,9 +505,6 @@ XAP_Preview_FontPreview::XAP_Preview_FontPreview(GR_Graphics * gc, const XML_Cha
 		m_iAscent(0),
 		m_iDescent(0),
 		m_iHeight(0)
-#ifdef WITH_PANGO
-		, m_pGlyphString(NULL)
-#endif
 {
 	if(pszClrBackground != NULL && strcmp(pszClrBackground,"transparent")!=0)
 		UT_parseColor(pszClrBackground,m_clrBackground);
@@ -517,14 +515,6 @@ XAP_Preview_FontPreview::XAP_Preview_FontPreview(GR_Graphics * gc, const XML_Cha
 
 XAP_Preview_FontPreview::~XAP_Preview_FontPreview()
 {
-#ifdef WITH_PANGO
-	if(m_pGlyphString)
-	{
-		g_list_foreach(m_pGlyphString, UT_free1PangoGlyphString, NULL);
-		g_list_free(m_pGlyphString);
-	}
-#endif
-	delete m_pFont;
 }
 
 /*!
@@ -654,40 +644,20 @@ void XAP_Preview_FontPreview::draw(void)
 	UT_sint32 iWinHeight = m_gc->tlu(getWindowHeight());
 	UT_sint32 iTop = (iWinHeight - m_iHeight)/2;
 	UT_sint32 len = UT_UCS4_strlen(m_pszChars);
-#ifndef WITH_PANGO
 	UT_sint32 twidth = m_gc->measureString(m_pszChars,0,len,NULL);
-#else
-	PangoRectangle ink_rect;
-	UT_uint32 twidth = 0;
-
-	if(!m_pGlyphString)
-		m_pGlyphString = m_gc->getPangoGlyphString(m_pszChars,len);
-
-	GList * pListItem = g_list_first(m_pGlyphString);
-	while (pListItem)
-	{
-		PangoGlyphString * pGString = (PangoGlyphString *) pListItem->data;
-		pango_glyph_string_extents(pGString, pFont, & ink_rect, NULL);
-		twidth += ink_rect.width;
-		pListItem = pListItem->next;
-	}
-#endif
-
 	UT_sint32 iLeft = (iWinWidth - twidth)/2;
 //
 // Fill the background color
 //
+	GR_Painter painter(m_gc);
+
 	if(pszBGColor)
-		m_gc->fillRect(BGcolor,iLeft,iTop,twidth,m_iHeight);
+		painter.fillRect(BGcolor,iLeft,iTop,twidth,m_iHeight);
 //
 // Do the draw chars at last!
 //
 	m_gc->setColor(FGcolor);
-#ifndef WITH_PANGO
-	m_gc->drawChars(m_pszChars, 0, len, iLeft, iTop);
-#else
-	m_gc->drawPangoGlyphString(m_pGlyphString, iLeft, iTop);
-#endif
+	painter.drawChars(m_pszChars, 0, len, iLeft, iTop);
 
 //
 // Do the decorations
@@ -695,27 +665,27 @@ void XAP_Preview_FontPreview::draw(void)
 	if(isUnder)
 	{
 		UT_sint32 iDrop = iTop + m_iAscent + m_iDescent/3;
-		m_gc->drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
+		painter.drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
 	}
 	if(isOver)
 	{
 		UT_sint32 iDrop = iTop + m_gc->tlu(1) + (UT_MAX(m_gc->tlu(10),m_iAscent) - m_gc->tlu(10))/8;
-		m_gc->drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
+		painter.drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
 	}
 	if(isStrike)
 	{
 		UT_sint32 iDrop = iTop + m_iAscent * 2 /3;
-		m_gc->drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
+		painter.drawLine(iLeft,iDrop,iLeft+twidth,iDrop);
 	}
 
 	// bad hardcoded color, but this will probably [ <-this assumption is the bad thing :) ] never be different anyway
 	m_gc->setColor(UT_RGBColor(0,0,0));
-	m_gc->drawLine(0, 0, m_gc->tlu(getWindowWidth()), 0);
-	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowWidth()) - m_gc->tlu(1),
+	painter.drawLine(0, 0, m_gc->tlu(getWindowWidth()), 0);
+	painter.drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowWidth()) - m_gc->tlu(1),
 		       m_gc->tlu(getWindowHeight()));
-	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0,
+	painter.drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0,
 		       m_gc->tlu(getWindowHeight()) - m_gc->tlu(1));
-	m_gc->drawLine(0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, 0);
+	painter.drawLine(0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, 0);
 }
 
 void XAP_Preview_FontPreview::clearScreen(void)
@@ -723,6 +693,8 @@ void XAP_Preview_FontPreview::clearScreen(void)
 	UT_sint32 iWidth = m_gc->tlu(getWindowWidth());
 	UT_sint32 iHeight = m_gc->tlu(getWindowHeight());
 
+	GR_Painter painter(m_gc);
+
 	// clear the whole drawing area, except for the border
-	m_gc->fillRect(m_clrBackground, 0 + m_gc->tlu(1), 0 + m_gc->tlu(1), iWidth - m_gc->tlu(2), iHeight - m_gc->tlu(2));
+	painter.fillRect(m_clrBackground, 0 + m_gc->tlu(1), 0 + m_gc->tlu(1), iWidth - m_gc->tlu(2), iHeight - m_gc->tlu(2));
 }
