@@ -20,39 +20,68 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-
 #include "ut_string.h"
+#include "ut_string_class.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_Language.h"
 #include "xap_Dlg_Language.h"
+#include "xap_Strings.h"   
+#include "xap_App.h"
+
+static int s_compareQ(const void * a, const void * b)                           
+{                                                                               
+	const XML_Char ** A = (const XML_Char **)(a);                                              
+	const XML_Char ** B = (const XML_Char **)(b);                                   	
+	return UT_strcoll(*A,*B);                                               
+}       
 
 /*****************************************************************/
 
 XAP_Dialog_Language::XAP_Dialog_Language(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialoglanguage")
+	: XAP_Dialog_NonPersistent(pDlgFactory,id, "interface/dialoglanguage"),
+	  m_bDocDefault(false)
 {
+	UT_uint32 nDontSort = 0, nSort = 0;
+	UT_uint32 i;	
+
 	m_answer		   = a_CANCEL;
 	m_pLanguage		   = NULL;
 	m_pLangProperty	   = NULL;
 	m_bChangedLanguage = false;
 	m_pLangTable = new UT_Language;
-
+	const XML_Char ** ppLanguagesTemp = new const XML_Char * [m_pLangTable->getCount()];	
+	
 	UT_ASSERT(m_pLangTable);
 	m_iLangCount = m_pLangTable->getCount();
-	m_ppLanguages = new const XML_Char * [m_iLangCount];
+	m_ppLanguages = new const XML_Char * [m_iLangCount];	
 	m_ppLanguagesCode = new const XML_Char * [m_iLangCount];
+	
+	for(i=0; i<m_iLangCount; i++)                                           
+	{                                                                       
+		if (m_pLangTable->getNthId(i)==XAP_STRING_ID_LANG_0) // Unsorted languages
+		{
+			m_ppLanguages[nDontSort]=m_pLangTable->getNthLangName(i);                                                    
+			nDontSort++;                                             
+		}
+		else
+		{
+			ppLanguagesTemp[nSort] = m_pLangTable->getNthLangName(i);                                                                     
+			nSort++;
+		}
+	}                                                                       
 
-	for(UT_uint32 i = 0; i < m_iLangCount; i++)
-	{
-		m_ppLanguages[i] = m_pLangTable->getNthLangName(i);
-		m_ppLanguagesCode[i] = m_pLangTable->getNthLangCode(i);
-	}
+	// sort the temporary array                                                                                                                      
+	qsort(ppLanguagesTemp, m_iLangCount-nDontSort, sizeof(XML_Char *), s_compareQ);
 
-	// Assign each language its code
+	  
+	// Copy the sorted codes and a ssign each language its code
 	for(UT_uint32 nLang = 0; nLang < m_iLangCount; nLang++)
 	{
-		for(UT_uint32 i = 0; i < m_iLangCount; i++)
+		if (nLang>=nDontSort)
+			m_ppLanguages[nLang]=ppLanguagesTemp[nLang-nDontSort];
+
+		for(i = 0; i < m_iLangCount; i++)
 		{
 			if (strcmp (m_ppLanguages[nLang], m_pLangTable->getNthLangName(i))==0)
 			{
@@ -61,6 +90,7 @@ XAP_Dialog_Language::XAP_Dialog_Language(XAP_DialogFactory * pDlgFactory, XAP_Di
 			}
 		}
 	}
+	delete [] ppLanguagesTemp;
 
 	// TODO: move spell-checking into XAP land and make this dialog
 	// TODO: more like the MSWord one (i.e. add):
@@ -107,7 +137,7 @@ XAP_Dialog_Language::tAnswer XAP_Dialog_Language::getAnswer(void) const
 
 bool XAP_Dialog_Language::getChangedLangProperty(const XML_Char ** pszLangProp) const
 {
-	UT_ASSERT(pszLangProp);
+	UT_return_val_if_fail(pszLangProp,false);
 	*pszLangProp = m_pLangProperty;
 	return m_bChangedLanguage;
 }
@@ -128,7 +158,7 @@ UT_Vector* XAP_Dialog_Language::getAvailableDictionaries()
 
 	for (UT_uint32 iItem = nItems; iItem; --iItem)
 	{
-		mapping = static_cast<DictionaryMapping*>(vec.getNthItem(iItem - 1));
+		mapping = static_cast<DictionaryMapping*>(const_cast<void*>(vec.getNthItem(iItem - 1)));
 
 		if (checker->doesDictionaryExist(mapping->lang.c_str()))
 			vecRslt->addItem( strdup(mapping->lang.c_str()));
@@ -137,4 +167,45 @@ UT_Vector* XAP_Dialog_Language::getAvailableDictionaries()
 	return vecRslt;
 }
 
+/*!
+    Fills s with the string to be used as a label for the the default language checkbox
+*/
+void XAP_Dialog_Language::getDocDefaultLangCheckboxLabel(UT_UTF8String &s)
+{
+	s.clear();
+	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+	UT_return_if_fail(pSS);
+
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_ULANG_DefaultLangChkbox, s);
+}
+
+/*!
+    Fills s with the string to be displayed in the default language static control
+*/
+void XAP_Dialog_Language::getDocDefaultLangDescription(UT_UTF8String & s)
+{
+	s.clear();
+	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+	UT_return_if_fail(pSS);
+
+	pSS->getValueUTF8(XAP_STRING_ID_DLG_ULANG_DefaultLangLabel, s);
+
+	s += m_docLang;
+}
+
+/*!
+    Initialises the dialogue to the current default lanaguage
+    pLang is a standard lang property (e.g., en-GB)
+ */
+void XAP_Dialog_Language::setDocumentLanguage(const XML_Char * pLang)
+{
+	UT_return_if_fail( pLang );
+	UT_return_if_fail(m_pLangTable);
+	
+	UT_uint32 indx = m_pLangTable->getIdFromCode(pLang);
+
+	// NB: m_docLang holds the translated language name, not the tag
+	XAP_App::getApp()->getStringSet()->getValueUTF8(indx, m_docLang);
+}
+
 
