@@ -25,7 +25,7 @@
 #include "ut_svg.h"
 #include "ut_string.h"
 #include "ut_bytebuf.h"
-#include "fl_Layout.h"
+#include "fl_ContainerLayout.h"
 #include "px_CR_Object.h"
 #include "pd_Document.h"
 #include "pp_AttrProp.h"
@@ -33,7 +33,7 @@
 #include "fg_GraphicVector.h"
 #include "ut_string_class.h"
 
-FG_Graphic* FG_GraphicVector::createFromChangeRecord(const fl_Layout* pFL,
+FG_Graphic* FG_GraphicVector::createFromChangeRecord(const fl_ContainerLayout* pFL,
 													 const PX_ChangeRecord_Object* pcro)
 {
 	FG_GraphicVector* pFG = new FG_GraphicVector();
@@ -47,9 +47,8 @@ FG_Graphic* FG_GraphicVector::createFromChangeRecord(const fl_Layout* pFL,
 	  for the image, and get the dataItem.  The bytes in the
 	  dataItem should be a SVG image.
 	*/
-	bool bFoundSpanAP = pFL->getSpanAttrProp(blockOffset, false,
-											 &pFG->m_pSpanAP);
-	if (bFoundSpanAP && pFG->m_pSpanAP)
+	pFL->getSpanAP(blockOffset, false, pFG->m_pSpanAP);
+	if (pFG->m_pSpanAP)
 	{
 		bool bFoundDataID = pFG->m_pSpanAP->getAttribute("dataid", pFG->m_pszDataID);
 		if (bFoundDataID && pFG->m_pszDataID)
@@ -65,6 +64,35 @@ FG_Graphic* FG_GraphicVector::createFromChangeRecord(const fl_Layout* pFL,
 }
 
 
+FG_Graphic* FG_GraphicVector::createFromStrux(const fl_ContainerLayout *pFL)
+{
+	FG_GraphicVector* pFG = new FG_GraphicVector();
+
+	bool bFoundDataItem = false;
+	const PD_Document* pDoc = pFL->getDocument();
+	/*
+	  Get the attribute list for this offset, lookup the dataid
+	  for the image, and get the dataItem.  The bytes in the
+	  dataItem should be a SVG image.
+	*/
+	pFL->getAP(pFG->m_pSpanAP);
+	
+	if (pFG->m_pSpanAP)
+	{
+		bool bFoundDataID = pFG->m_pSpanAP->getAttribute(PT_STRUX_IMAGE_DATAID, pFG->m_pszDataID);
+		if (bFoundDataID && pFG->m_pszDataID)
+		{
+			bFoundDataItem = pDoc->getDataItemDataByName(static_cast<const char*>(pFG->m_pszDataID), const_cast<const UT_ByteBuf **>(&pFG->m_pbbSVG), NULL, NULL);
+		}
+	}
+
+	if (!bFoundDataItem)
+		DELETEP(pFG);
+
+	return pFG;
+}
+
+
 FG_GraphicVector::FG_GraphicVector()
 {
 	m_pbbSVG = NULL;
@@ -81,6 +109,19 @@ FG_GraphicVector::~FG_GraphicVector()
 		m_pbbSVG = NULL;
 }
 
+FG_Graphic * FG_GraphicVector::clone(void)
+{
+	FG_GraphicVector * pClone = new FG_GraphicVector();
+	pClone->m_pbbSVG = m_pbbSVG;
+	pClone->m_pSpanAP = m_pSpanAP;
+	pClone->m_pszDataID = m_pszDataID;
+	pClone->m_iWidth = m_iWidth; 
+	pClone->m_iHeight = m_iHeight;
+	pClone->m_iMaxW = m_iMaxW;
+	pClone->m_iMaxH = m_iMaxH;
+	return static_cast<FG_Graphic *>(pClone);
+}
+
 FGType FG_GraphicVector::getType(void)
 {
 	return FGT_Vector;
@@ -130,6 +171,14 @@ const char * FG_GraphicVector::getHeightProp(void)
 	return szHeight;
 }
 
+/*!
+ * If either the graphics class or graphics resolution changes, regenerate
+ * the image.
+ */
+GR_Image * FG_GraphicVector::regenerateImage(GR_Graphics * pG)
+{
+	return generateImage(pG,m_pSpanAP,m_iMaxW,m_iMaxH);
+}
 
 //
 //  We will generate an image at the proper resolution for display in the
@@ -174,7 +223,8 @@ GR_Image* FG_GraphicVector::generateImage(GR_Graphics* pG,
 
 	if (maxW != 0 && iDisplayWidth > maxW) iDisplayWidth = maxW;
 	if (maxH != 0 && iDisplayHeight > maxH) iDisplayHeight = maxH;
-
+	m_iMaxH = maxH;
+	m_iMaxW = maxW;
 	GR_Image *pImage = pG->createNewImage(static_cast<const char*>(m_pszDataID), m_pbbSVG, iDisplayWidth, iDisplayHeight, GR_Image::GRT_Vector);
 	return pImage;
 }
@@ -194,7 +244,7 @@ UT_Error FG_GraphicVector::insertIntoDocument(PD_Document* pDoc, UT_uint32 res,
 	/*
 	  Create the data item
 	*/
-	char * mimetype = UT_strdup("image/svg-xml");
+	char * mimetype = UT_strdup("image/svg+xml");
    	pDoc->createDataItem(szName, false, m_pbbSVG, mimetype, NULL);
 
 	UT_String szProps;
@@ -228,6 +278,61 @@ UT_Error FG_GraphicVector::insertIntoDocument(PD_Document* pDoc, UT_uint32 res,
 	return UT_OK;
 }
 
+/*!
+ * Insert an image at the strux given. This will become the
+ * background image for the container defined by the strux.
+ */
+UT_Error FG_GraphicVector::insertAtStrux(PD_Document* pDoc, 
+										 UT_uint32 res,
+										 UT_uint32 iPos,
+										 PTStruxType iStruxType,
+										 const char* szName)
+{
+	UT_ASSERT(pDoc);
+	UT_ASSERT(szName);
+
+	/*
+	  Create the data item
+	*/
+	char * mimetype = UT_strdup("image/svg+xml");
+   	pDoc->createDataItem(szName, false, m_pbbSVG, mimetype, NULL);
+
+
+	/*
+	  Insert the object into the document.
+	*/
+	UT_String szProps;
+
+	szProps += "width:";
+	szProps += UT_convertInchesToDimensionString(DIM_IN, static_cast<double>(m_iWidth)/res, "3.2");
+	szProps += "; height:";
+	szProps += UT_convertInchesToDimensionString(DIM_IN, static_cast<double>(m_iHeight)/res, "3.2");
+
+#ifndef __MRC__
+	const XML_Char*	attributes[] = {
+		PT_STRUX_IMAGE_DATAID, szName,
+		PT_PROPS_ATTRIBUTE_NAME, szProps.c_str(),
+	   	NULL, NULL
+	};
+#else
+	// MrCPP does not like the above
+	const XML_Char * attributes[] = {
+		PT_STRUX_IMAGE_DATAID, NULL,
+		PT_PROPS_ATTRIBUTE_NAME, NULL,
+	   	NULL, NULL
+	};
+	attributes [1] = szName;
+	attributes [3] = szProps;
+#endif
+
+	pDoc->changeStruxFmt(PTC_AddFmt,iPos,iPos,attributes,NULL,iStruxType);
+
+
+	// TODO: better error checking in this function
+	return UT_OK;
+}
+
+
 bool FG_GraphicVector::setVector_SVG(UT_ByteBuf* pBB)
 {
 	if (m_bOwnSVG)
