@@ -34,6 +34,7 @@
 #include "fl_Squiggles.h"
 #include "fl_AutoNum.h"
 #include "fp_Page.h"
+#include "fp_Line.h"
 #include "fp_TextRun.h"
 #include "fp_Run.h"
 #include "fv_View.h"
@@ -588,6 +589,53 @@ void FL_DocLayout::fillLayouts(void)
 			}
 		}
 	}
+	if(m_pView)
+	{
+	        fl_DocSectionLayout * pLastSec = getLastSection();
+		fl_ContainerLayout * pCL = pLastSec->getLastLayout();
+		fl_BlockLayout * pBL = NULL;
+		bool bRebreak = false;
+		if(pCL->getContainerType() == FL_CONTAINER_BLOCK)
+		{
+	              pBL = static_cast<fl_BlockLayout *>(pCL);
+		}
+		else
+		{
+	              pBL = pCL->getPrevBlockInDocument();
+		}
+		if(pBL)
+		{
+		      fp_Line * pLine = static_cast<fp_Line *>(pBL->getLastContainer());
+		      if(pLine == NULL)
+		      {
+			    bRebreak = true;
+		      }
+		      else if(pLine->getPage() == NULL)
+		      {
+			    bRebreak = true;
+		      }
+		      else
+		      {
+			    fp_Page * pPage = getFirstPage();
+			    while(pPage && pPage != pLine->getPage())
+			    {
+			         pPage = pPage->getNext();
+			    }
+			    if(pLine->getPage() != pPage)
+			    {
+			         bRebreak = true;
+			    }
+			    if(pLine->getPage() == getFirstPage())
+			    {
+				bRebreak = true;
+			    }
+		      }
+		}
+		if(bRebreak)
+		{
+		      getFirstSection()->completeBreakSection();
+		}
+	}
 }
 
 
@@ -662,35 +710,75 @@ void FL_DocLayout::getStringFromFootnoteVal(UT_String & sVal, UT_sint32 iVal, Fo
 		UT_String_sprintf (sVal,"%d)", iVal);
 		break;
 	case FOOTNOTE_TYPE_LOWER:
-		UT_String_sprintf (sVal,"%s",autoCalc.dec2ascii(iVal,96));
+	{
+		char * val = autoCalc.dec2ascii(iVal,96);
+		UT_String_sprintf (sVal,"%s",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_LOWER_PAREN:
-		UT_String_sprintf (sVal,"(%s)",autoCalc.dec2ascii(iVal,96));
+	{
+		char * val = autoCalc.dec2ascii(iVal,96);
+		UT_String_sprintf (sVal,"(%s)",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_LOWER_OPEN_PAREN:
-		UT_String_sprintf (sVal,"%s)",autoCalc.dec2ascii(iVal,96));
+	{
+		char * val = autoCalc.dec2ascii(iVal,96);
+		UT_String_sprintf (sVal,"%s)",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_UPPER:
-		UT_String_sprintf (sVal,"%s",autoCalc.dec2ascii(iVal,64));
+	{
+		char * val = autoCalc.dec2ascii(iVal,64);
+		UT_String_sprintf (sVal,"%s",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_UPPER_PAREN:
-		UT_String_sprintf (sVal,"(%s)",autoCalc.dec2ascii(iVal,64));
+	{
+		char * val = autoCalc.dec2ascii(iVal,64);
+		UT_String_sprintf (sVal,"(%s)",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_UPPER_OPEN_PAREN:
-		UT_String_sprintf (sVal,"%s)",autoCalc.dec2ascii(iVal,64));
+	{
+		char * val = autoCalc.dec2ascii(iVal,64);
+		UT_String_sprintf (sVal,"%s)",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_LOWER_ROMAN:
-		UT_String_sprintf (sVal,"%s",autoCalc.dec2roman(iVal,true));
+	{
+		char * val = autoCalc.dec2roman(iVal,true);
+		UT_String_sprintf (sVal,"%s",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_LOWER_ROMAN_PAREN:
-		UT_String_sprintf (sVal,"(%s)",autoCalc.dec2roman(iVal,true));
+	{
+		char * val = autoCalc.dec2roman(iVal,true);
+		UT_String_sprintf (sVal,"(%s)",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_UPPER_ROMAN:
-		UT_String_sprintf (sVal,"%s",autoCalc.dec2roman(iVal,false));
+	{
+		char * val = autoCalc.dec2roman(iVal,false);
+		UT_String_sprintf (sVal,"%s",val);
+		FREEP(val);
 		break;
+	}
 	case FOOTNOTE_TYPE_UPPER_ROMAN_PAREN:
-		UT_String_sprintf (sVal,"(%s)",autoCalc.dec2roman(iVal,false));
+	{
+		char * val = autoCalc.dec2roman(iVal,false);
+		UT_String_sprintf (sVal,"(%s)",val);
+		FREEP(val);
 		break;
+	}
 	default:
 		UT_String_sprintf (sVal,"%d", iVal);
 	}
@@ -2143,27 +2231,7 @@ FL_DocLayout::_toggleAutoSpell(bool bSpell)
 	if (bSpell)
 	{
 		UT_DEBUGMSG(("Rechecking spelling in blocks \n"));
-		// When enabling, recheck the whole document
-		fl_DocSectionLayout * pSL = getFirstSection();
-		if(pSL)
-		{
-			fl_ContainerLayout* b = pSL->getFirstLayout();
-			while (b)
-			{
-				// TODO: just check and remove matching squiggles
-				// for now, destructively recheck the whole thing
-				if(b->getContainerType() == FL_CONTAINER_BLOCK)
-				{
-					UT_DEBUGMSG(("Add block  %x for spellcheck \n",b));
-					queueBlockForBackgroundCheck(bgcrSpelling, static_cast<fl_BlockLayout *>(b));
-					b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
-				}
-				else
-				{
-					b = b->getNext();
-				}
-			}
-		}
+		queueAll(bgcrSpelling);
 	}
 	else
 	{
@@ -2232,27 +2300,7 @@ FL_DocLayout::_toggleAutoGrammar(bool bGrammar)
 	if (bGrammar)
 	{
 		UT_DEBUGMSG(("Rechecking Grammar in blocks \n"));
-		// When enabling, recheck the whole document
-		fl_DocSectionLayout * pSL = getFirstSection();
-		if(pSL)
-		{
-			fl_ContainerLayout* b = pSL->getFirstLayout();
-			while (b)
-			{
-				// TODO: just check and remove matching squiggles
-				// for now, destructively recheck the whole thing
-				if(b->getContainerType() == FL_CONTAINER_BLOCK)
-				{
-					UT_DEBUGMSG(("Add block  %x for Grammar check \n",b));
-					queueBlockForBackgroundCheck(bgcrGrammar, static_cast<fl_BlockLayout *>(b));
-					b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
-				}
-				else
-				{
-					b = b->getNext();
-				}
-			}
-		}
+		queueAll(bgcrGrammar);
 	}
 	else
 	{
@@ -2568,6 +2616,55 @@ void FL_DocLayout::dequeueAll(void)
 	}
 }
 
+void FL_DocLayout::queueAll(UT_uint32 iReason)
+{
+	fl_DocSectionLayout * pSL = getFirstSection();
+	if(pSL)
+	{
+		// We will place the block that contains the ins point and its immediate neigbours
+		// at the top of the queue, this will make the check look faster to the user
+		FV_View * pView = getView();
+		UT_GenericVector<fl_BlockLayout*> vBL;
+		const UT_sint32 iLimit = 5;
+		
+		fl_BlockLayout * pCurBL = pView->getBlockAtPosition(pView->getPoint());
+
+		if(pCurBL)
+		{
+			fl_BlockLayout * pBL = pCurBL;
+
+			UT_sint32 i = 0;
+			for(i = 0; i < iLimit/2 + iLimit%2 && pBL; ++i, pBL = pBL->getPrevBlockInDocument())
+			{
+				vBL.addItem(pBL);
+			}
+
+			pBL = pCurBL->getNextBlockInDocument();
+			for(i = iLimit/2 + iLimit%2; i < iLimit && pBL; ++i, pBL = pBL->getNextBlockInDocument())
+			{
+				vBL.addItem(pBL);
+			}
+		}
+		
+		fl_ContainerLayout* b = pSL->getFirstLayout();
+		while (b)
+		{
+			// TODO: just check and remove matching squiggles
+			// for now, destructively recheck the whole thing
+			if(b->getContainerType() == FL_CONTAINER_BLOCK)
+			{
+				bool bHead = (vBL.findItem(static_cast<fl_BlockLayout *>(b)) >= 0);
+				queueBlockForBackgroundCheck(iReason, static_cast<fl_BlockLayout *>(b), bHead);
+				b = static_cast<fl_BlockLayout *>(b)->getNextBlockInDocument();
+			}
+			else
+			{
+				b = b->getNext();
+			}
+		}
+	}
+}
+
 /*!
  * Set the next block to be grammar checked. It won't actually get checked
  * until the insertPoint leaves this block.
