@@ -26,22 +26,37 @@
 #include "pt_Types.h"
 #include "fl_SectionLayout.h"
 #include "fp_PageSize.h"
-
+#include "fp_ContainerObject.h"
 
 class FL_DocLayout;
 class fp_Column;
 class fp_Container;
 class fp_ShadowContainer;
 class fp_FootnoteContainer;
+class fp_FrameContainer;
 class fl_DocSectionLayout;
 class fl_HdrFtrSectionLayout;
 class fl_HdrFtrShadow;
 class fl_FootnoteLayout;
+class fl_FrameLayout;
 class FV_View;
 class GR_Graphics;
 class fp_TableContainer;
+class fl_TOCLayout;
 struct dg_DrawArgs;
 
+class _BL
+	{
+	public:
+		_BL(fl_BlockLayout * pBL,fp_Line * pL) :
+			m_pBL(pBL),
+			m_pL(pL)
+			{
+			}
+		fl_BlockLayout * m_pBL;
+		fp_Line * m_pL;
+	};
+
 // ----------------------------------------------------------------
 class ABI_EXPORT fp_Page
 {
@@ -65,11 +80,13 @@ public:
 	UT_sint32			getColumnGap(void) const;
 	FL_DocLayout*		getDocLayout() const;
 	void				setView(FV_View*);
+    bool                isOnScreen(void);
 
 	inline fl_DocSectionLayout* getOwningSection(void) const { return m_pOwner; }
 
 	PT_DocPosition		getFirstLastPos(bool bFirst) const;
-	void				mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL);
+	void				mapXYToPosition(bool bNotFrames,UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL,bool & isTOC, bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL);
+	void				mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, bool& bBOL, bool& bEOL, bool & isTOC,bool bUseHdrFtr = false, fl_HdrFtrShadow ** pShadow = NULL);
 	void				getScreenOffsets(fp_Container*, UT_sint32& xoff, UT_sint32& yoff) const;
 
 	void				draw(dg_DrawArgs*, bool bAlaysUseWhiteBackground=false);
@@ -87,9 +104,8 @@ public:
 	bool				insertColumnLeader(fp_Column* pLeader, fp_Column* pAfter);
 	void				removeColumnLeader(fp_Column* pLeader);
 	bool				isEmpty(void) const;
-
+	fp_Container *      updatePageForWrapping(fp_Column *& pNextCol);
 	// Header/Footer functions.
-	void 				footnoteHeightChanged(void);
 	void                removeHdrFtr(HdrFtrType hfType);
 	fp_ShadowContainer* getHdrFtrP(HdrFtrType hfType) const;
 	fp_ShadowContainer*	getHdrFtrContainer(fl_HdrFtrSectionLayout*);
@@ -97,6 +113,7 @@ public:
 											 HdrFtrType hfType);
 
 	// Footnote functions.
+	void 				footnoteHeightChanged(void);
 	UT_uint32			countFootnoteContainers(void) const;
 	fp_FootnoteContainer* getNthFootnoteContainer(UT_sint32 n) const; 
 	bool				insertFootnoteContainer(fp_FootnoteContainer * pFC);
@@ -104,6 +121,29 @@ public:
 	UT_sint32           findFootnoteContainer(fp_FootnoteContainer * pFC);
 	void                clearScreenFootnotes(void);
 	UT_sint32           getFootnoteHeight(void);
+
+	// Frame functions.
+	void 				frameHeightChanged(void);
+	UT_uint32			countAboveFrameContainers(void) const;
+	UT_uint32			countBelowFrameContainers(void) const;
+	fp_FrameContainer*  getNthAboveFrameContainer(UT_sint32 n) const; 
+	fp_FrameContainer*  getNthBelowFrameContainer(UT_sint32 n) const; 
+	bool				insertFrameContainer(fp_FrameContainer * pFC);
+	void				removeFrameContainer(fp_FrameContainer * pFC);
+	UT_sint32           findFrameContainer(fp_FrameContainer * pFC);
+	void                clearScreenFrames(void);
+	void                markDirtyOverlappingRuns(fp_FrameContainer * pFC);
+    void                expandDamageRect(UT_sint32 x, UT_sint32 y, 
+										 UT_sint32 width, UT_sint32 height);
+	void                redrawDamagedFrames(dg_DrawArgs* pDA);
+	bool                overlapsWrappedFrame(fp_Line * pLine);
+	bool                overlapsWrappedFrame(UT_Rect & rec);
+	bool                TopBotMarginChanged(void);
+	void                setLastMappedTOC(fl_TOCLayout * pTOCL)
+		{ m_pLastMappedTOC = pTOCL;}
+	fl_TOCLayout *      getLastMappedTOC(void)
+		{ return m_pLastMappedTOC;}
+	fg_FillType *       getFillType(void);
 #ifdef FMT_TEST
 	void				__dump(FILE * fp) const;
 #endif
@@ -129,14 +169,21 @@ private:
 
 	bool				m_bNeedsRedraw;
 
-	UT_Vector			m_vecColumnLeaders;
+	UT_GenericVector<fp_Column *> m_vecColumnLeaders;
 
 	fl_DocSectionLayout*	m_pOwner;
 
 	fp_ShadowContainer* m_pFooter;
 	fp_ShadowContainer* m_pHeader;
 
-	UT_Vector			m_vecFootnotes;
+	UT_GenericVector<fp_FootnoteContainer *> m_vecFootnotes;
+	fg_FillType         m_FillType;
+	UT_GenericVector<fp_FrameContainer *> m_vecAboveFrames;
+	UT_GenericVector<fp_FrameContainer *> m_vecBelowFrames;
+	fl_TOCLayout *      m_pLastMappedTOC;
+
+	UT_Rect             m_rDamageRect;
+	UT_sint32           m_iCountWrapPasses;
 };
 
 #endif /* PAGE_H */
