@@ -42,62 +42,129 @@
 
 /****************************************************************/
 /****************************************************************/
+
+/*!
+    A helper function which translates a revision attribute associated with fragment of type pts at
+    pos dpos into arrays of attributes and properties suitable for passing into formating and other funcitons.
+
+    Revisions -- an instance of an empty PP_RevisionsAttr (i.e., PP_RevisionsAttr(NULL);)
+    
+    ppRevAttrib -- pointers to arrays of attributes and properties; the actual props and attribs are
+    ppRevProps     found inside the Revisions variable, so the returned pointers are only valid
+                   within the Revisions scope !!!.
+
+    ppAttrib -- pointers to any attributes/properties that are to be added to this revision, can be NULL
+    ppProps
+*/
+bool pt_PieceTable::_translateRevisionAttribute(PP_RevisionAttr & Revisions, PT_AttrPropIndex indexAP,
+												PP_RevisionType eType,
+												const XML_Char ** & ppRevAttrib,
+												const XML_Char ** & ppRevProps,
+												const XML_Char **   ppAttrib,
+												const XML_Char **   ppProps)
+{
+	// foolproofing
+	ppRevAttrib = NULL;
+	ppRevProps = NULL;
+	
+	UT_return_val_if_fail(m_pDocument->isMarkRevisions(),false );
+	
+	const PP_AttrProp * pRevisedAP = NULL;
+	const PP_AttrProp * pAP = NULL;
+	getAttrProp(indexAP, &pAP);
+	const XML_Char name[] = "revision";
+
+	if(pAP)
+	{
+		const XML_Char * pRev = NULL;
+		if(pAP->getAttribute(name, pRev))
+		{
+			// OK, the previous strux had a revision attribute, which was copied into the new
+			// strux. This revision attribute can contain significant properties and attributes
+			// which need to be preserved (such as list ids). However, we want the revision
+			// attributes from the addition and fmt records to be transfered into the regular
+			// attrs and props
+				
+			Revisions.setRevision(pRev);
+			Revisions.pruneForCumulativeResult(m_pDocument);
+			pRevisedAP = Revisions.getLastRevision();
+
+			// it is legal of pRevisedAP to be NULL here -- it simply means that the cumulative
+			// effect of the revisions attribute was nothing at all (i.e., the highest revision
+			// was a deletion)
+			if(pRevisedAP)
+			{
+				PP_RevisionAttr Revisions2(NULL);
+
+				// now add the revision attribute
+				Revisions2.addRevision(m_pDocument->getRevisionId(),eType,ppAttrib,ppProps);
+				const_cast<PP_AttrProp*>(pRevisedAP)->setAttribute(name, Revisions2.getXMLstring());
+			}
+			
+		}
+	}
+	
+	if(!pRevisedAP)
+	{
+		// there was either no pAP or no pRev, just add the current revision ...
+		// we need to create a rev. instance in Revisions
+		Revisions.addRevision(m_pDocument->getRevisionId(),eType,ppAttrib,ppProps);
+		pRevisedAP = Revisions.getLastRevision();
+		UT_return_val_if_fail( pRevisedAP, false );
+
+		// now set the revision attribute of the revision
+		const_cast<PP_AttrProp*>(pRevisedAP)->setAttribute(name, Revisions.getXMLstring());
+	}
+
+	ppRevAttrib = pRevisedAP->getAttributes();
+	ppRevProps  = pRevisedAP->getProperties();
+	
+	return true;
+}
+
+
 bool pt_PieceTable::insertStrux(PT_DocPosition dpos,
 								PTStruxType pts,
 								pf_Frag_Strux ** ppfs_ret)
 {
-	if(!_realInsertStrux(dpos,pts,0,0,ppfs_ret))
-		return false;
-
 	if(m_pDocument->isMarkRevisions())
 	{
-		PP_RevisionAttr Revisions(NULL);
-		Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_ADDITION,NULL,NULL);
-
-		const XML_Char name[] = "revision";
-		const XML_Char * ppRevAttrib[3];
-		ppRevAttrib[0] = name;
-		ppRevAttrib[1] = Revisions.getXMLstring();
-		ppRevAttrib[2] = NULL;
-
-		UT_uint32 iLen;
-
-		switch (pts)
+		// when the strux is inserted in non-revision mode, it inherits the AP from the previous
+		// strux. In revision mode this does not necessarily work because we may need to have a
+		// different revision attribute. Consequently, we need to ensure that the AP that gets
+		// assigned to the new strux contains all relevant attrs and props from the AP of the
+		// previous strux -- we do this by obtaining the index of the AP of the previous strux and
+		// running it through _translateRevisionAttribute() which will gives back all attrs and
+		// props that need to be passed to _realInsertStrux()
+		pf_Frag_Strux * pfsContainer = NULL;
+		bool bFoundContainer = _getStruxFromPosition(dpos,&pfsContainer); // the orig. strux
+		UT_return_val_if_fail(bFoundContainer, false);
+	
+		if(isEndFootnote(pfsContainer))
 		{
-			case PTX_Block:
-				iLen = pf_FRAG_STRUX_BLOCK_LENGTH;
-				break;
-
-			case PTX_Section:
-			case PTX_SectionHdrFtr:
-			case PTX_SectionEndnote:
-			case PTX_SectionTable:
-			case PTX_SectionCell:
-			case PTX_SectionFootnote:
-			case PTX_SectionMarginnote:
-			case PTX_SectionFrame:
-			case PTX_EndCell:
-			case PTX_EndTable:
-			case PTX_EndFootnote:
-			case PTX_EndEndnote:
-			case PTX_EndMarginnote:
-			case PTX_EndFrame:
-				iLen = pf_FRAG_STRUX_SECTION_LENGTH;
-				break;
-
-			default:
-				UT_ASSERT(UT_NOT_IMPLEMENTED);
-				iLen = 1;
-				break;
+			bFoundContainer = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+			UT_return_val_if_fail(bFoundContainer, false);
 		}
 
-		dpos += iLen; // we want to change the format of the new
-					  // strux, not the old one
+		PT_AttrPropIndex indexAP = 0;
+		if (pfsContainer->getStruxType() == pts)
+		{
+			indexAP = pfsContainer->getIndexAP();
+		}
 
-		return _realChangeStruxFmt(PTC_AddFmt, dpos, dpos + iLen, ppRevAttrib,NULL,pts);
+		PP_RevisionAttr Revisions(NULL);
+		const XML_Char ** ppRevAttrib = NULL;
+		const XML_Char ** ppRevProps  = NULL;
+
+		_translateRevisionAttribute(Revisions, indexAP, PP_REVISION_ADDITION, ppRevAttrib, ppRevProps, 0, 0);
+
+		//return _realChangeStruxFmt(PTC_AddFmt, dpos, dpos + iLen, ppRevAttrib,ppRevProps,pts);
+		return _realInsertStrux(dpos,pts,ppRevAttrib,ppRevProps,ppfs_ret);
+	}
+	else
+	{
+		return _realInsertStrux(dpos,pts,0,0,ppfs_ret);
 	}
-
-	return true;
 }
 
 bool pt_PieceTable::insertStrux(PT_DocPosition dpos,
@@ -106,59 +173,71 @@ bool pt_PieceTable::insertStrux(PT_DocPosition dpos,
 								const XML_Char ** properties,
 								pf_Frag_Strux ** ppfs_ret)
 {
-
-	if(!_realInsertStrux(dpos,pts,attributes,properties,ppfs_ret))
-		return false;
-
 	if(m_pDocument->isMarkRevisions())
 	{
-		PP_RevisionAttr Revisions(NULL);
-		Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_ADDITION,attributes,properties);
-
-		const XML_Char name[] = "revision";
-		const XML_Char * ppRevAttrib[3];
-		ppRevAttrib[0] = name;
-		ppRevAttrib[1] = Revisions.getXMLstring();
-		ppRevAttrib[2] = NULL;
-
-		UT_uint32 iLen;
-
-		switch (pts)
+		// This is just like the previous method, except that in addition to calling
+		// _translateRevisionAttribute() we also need to set the attrs and props
+		// passed to us.
+		pf_Frag_Strux * pfsContainer = NULL;
+		bool bFoundContainer = _getStruxFromPosition(dpos,&pfsContainer); // the orig. strux
+		UT_return_val_if_fail(bFoundContainer, false);
+	
+		if(isEndFootnote(pfsContainer))
 		{
-			case PTX_Block:
-				iLen = pf_FRAG_STRUX_BLOCK_LENGTH;
-				break;
-
-			case PTX_Section:
-			case PTX_SectionHdrFtr:
-			case PTX_SectionEndnote:
-			case PTX_SectionTable:
-			case PTX_SectionCell:
-			case PTX_SectionFootnote:
-			case PTX_SectionMarginnote:
-			case PTX_SectionFrame:
-			case PTX_EndCell:
-			case PTX_EndTable:
-			case PTX_EndFootnote:
-			case PTX_EndEndnote:
-			case PTX_EndMarginnote:
-			case PTX_EndFrame:
-				iLen = pf_FRAG_STRUX_SECTION_LENGTH;
-				break;
-
-			default:
-				UT_ASSERT(UT_NOT_IMPLEMENTED);
-				iLen = 1;
-				break;
+			bFoundContainer = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+			UT_return_val_if_fail(bFoundContainer, false);
 		}
 
-		dpos += iLen; // we want to change the format of the new
-					  // strux, not the old one
+		PT_AttrPropIndex indexAP = 0;
+		if (pfsContainer->getStruxType() == pts)
+		{
+			indexAP = pfsContainer->getIndexAP();
+		}
 
-		return _realChangeStruxFmt(PTC_AddFmt, dpos, dpos + iLen, ppRevAttrib,NULL,pts);
+		PP_RevisionAttr Revisions(NULL);
+		const XML_Char ** ppRevAttrs = NULL;
+		const XML_Char ** ppRevProps  = NULL;
+
+		_translateRevisionAttribute(Revisions, indexAP, PP_REVISION_ADDITION,
+									ppRevAttrs, ppRevProps, NULL, NULL);
+
+		// count original attributes and the revision-inherited attributes
+		// and add them to the revision attribute
+		UT_uint32 iAttrCount = 0;
+		for (; attributes && attributes[iAttrCount]; iAttrCount+=2){}
+
+		UT_uint32 iRevAttrCount = 0;
+		for (; ppRevAttrs && ppRevAttrs[iRevAttrCount]; iRevAttrCount+=2){}
+
+		const XML_Char ** ppRevAttrib = NULL;
+		if(iAttrCount + iRevAttrCount > 0)
+		{
+			ppRevAttrib = new const XML_Char * [iAttrCount + iRevAttrCount + 1];
+			UT_return_val_if_fail( ppRevAttrib, false );
+
+			UT_uint32 i = 0;
+			for (i = 0; i < iAttrCount; ++i)
+			{
+				ppRevAttrib[i] = attributes[i];
+			}
+
+			for (; i < iRevAttrCount + iAttrCount; ++i)
+			{
+				ppRevAttrib[i] = ppRevAttrs[i - iAttrCount];
+			}
+		
+			ppRevAttrib[i]   = NULL;
+		}
+		
+		//return _realChangeStruxFmt(PTC_AddFmt, dpos, dpos + iLen, ppRevAttrib,NULL,pts);
+		bool bRet = _realInsertStrux(dpos,pts,ppRevAttrib,properties,ppfs_ret);
+		delete [] ppRevAttrib;
+		return bRet;
+	}
+	else
+	{
+		return _realInsertStrux(dpos,pts,attributes,properties,ppfs_ret);
 	}
-
-	return true;
 }
 
 
@@ -196,12 +275,23 @@ bool pt_PieceTable::_createStrux(PTStruxType pts,
 		pfs = new pf_Frag_Strux_SectionEndnote(this, indexAP);
 		break;
 
+	case PTX_SectionFrame:
+		pfs = new pf_Frag_Strux_SectionFrame(this, indexAP);
+		break;
+
 	case PTX_SectionTable:
 		pfs = new pf_Frag_Strux_SectionTable(this, indexAP);
 		break;
+
+	case PTX_EndFrame:
+		pfs = new pf_Frag_Strux_SectionEndFrame(this, indexAP);
+		break;
 	case PTX_SectionCell:
 		pfs = new pf_Frag_Strux_SectionCell(this, indexAP);
 		break;
+	case PTX_SectionTOC:
+		pfs = new pf_Frag_Strux_SectionTOC(this, indexAP);
+		break;
 	case PTX_EndTable:
 		pfs = new pf_Frag_Strux_SectionEndTable(this, indexAP);
 		break;
@@ -214,9 +304,12 @@ bool pt_PieceTable::_createStrux(PTStruxType pts,
 	case PTX_EndEndnote:
 		pfs = new pf_Frag_Strux_SectionEndEndnote(this, indexAP);
 		break;
+	case PTX_EndTOC:
+		pfs = new pf_Frag_Strux_SectionEndTOC(this, indexAP);
+		break;
 
 	default:
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 		break;
 	}
 
@@ -241,10 +334,32 @@ void pt_PieceTable::_insertStrux(pf_Frag * pf,
 								 pf_Frag_Strux * pfsNew)
 {
 	// insert the new strux frag at (pf,fragOffset)
+	//
+	// In the case of Frames, the frame must be placed just before the next
+	// strux (that's not an emebedded-type strux
+	//
+
+	if(pfsNew->getStruxType() == PTX_SectionFrame)
+	{
+		pf_Frag_Strux * pfsNext = NULL;
+		if(pf->getType() != pf_Frag::PFT_Strux)
+		{
+			_getNextStruxAfterFragSkip(pf, &pfsNext);
+			if(pfsNext != NULL)
+			{
+				pf = static_cast<pf_Frag *>(pfsNext);
+			}
+			if(isEndFootnote(pf))
+			{
+				pf = pf->getNext();
+			}
+			fragOffset = 0;
+		}
+	}
 	switch (pf->getType())
 	{
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return;
 
 	case pf_Frag::PFT_Object:
@@ -254,7 +369,7 @@ void pt_PieceTable::_insertStrux(pf_Frag * pf,
 			// insert pfsNew before pf.
 			// TODO this may introduce some oddities due to empty paragraphs.
 			// TODO investigate this later.
-			UT_ASSERT(fragOffset == 0);
+			UT_return_if_fail (fragOffset == 0);
 //
 // OK find the real container strux.
 //
@@ -267,7 +382,7 @@ void pt_PieceTable::_insertStrux(pf_Frag * pf,
 			// insert pfsNew after pf.
             // before this.
 			// TODO check this.
-			UT_ASSERT(fragOffset == 0);
+			UT_return_if_fail (fragOffset == 0);
 			m_fragments.insertFrag(pf,pfsNew);
 			return;
 		}
@@ -315,7 +430,7 @@ void pt_PieceTable::_insertStrux(pf_Frag * pf,
 				UT_uint32 lenTail = pft->getLength() - fragOffset;
 				PT_BufIndex biTail = m_varset.getBufIndex(pft->getBufIndex(),fragOffset);
 				pf_Frag_Text * pftTail = new pf_Frag_Text(this,biTail,lenTail,pft->getIndexAP(),pft->getField());
-				UT_ASSERT(pftTail);
+				UT_return_if_fail (pftTail);
 
 				pft->changeLength(fragOffset);
 				m_fragments.insertFrag(pft,pfsNew);
@@ -342,7 +457,7 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 	// a table.  this function can only be called while editing the
 	// document.
 
-	UT_ASSERT(m_pts==PTS_Editing);
+	UT_return_val_if_fail (m_pts==PTS_Editing, false);
 
 	// get the fragment at the doc postion containing the given
 	// document position.
@@ -350,13 +465,21 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 	pf_Frag * pf = NULL;
 	PT_BlockOffset fragOffset = 0;
 	bool bFoundFrag = getFragFromPosition(dpos,&pf,&fragOffset);
-	UT_ASSERT(bFoundFrag);
+	UT_return_val_if_fail (bFoundFrag, false);
 
 	// get the strux containing the given position.
 
 	pf_Frag_Strux * pfsContainer = NULL;
 	bool bFoundContainer = _getStruxFromPosition(dpos,&pfsContainer);
-	UT_ASSERT(bFoundContainer);
+	UT_return_val_if_fail (bFoundContainer,false);
+	//
+	// Can only insert an endTOC into a TOC
+	//
+	if((pfsContainer->getStruxType() == PTX_SectionTOC) && (pts != PTX_EndTOC))
+	{
+		bFoundContainer = _getStruxFromPosition(pfsContainer->getPos(),&pfsContainer);
+		dpos--;
+	}
 	if(isEndFootnote(pfsContainer))
 	{
 		bFoundContainer = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
@@ -377,6 +500,52 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 		indexAP = pfsContainer->getIndexAP();
 	}
 
+//
+// Look to see if we're in the middle of a hyperlink span now.
+//
+	pf_Frag * pHype = _findPrevHyperlink(pf);
+	if(pHype != NULL && (pts != PTX_SectionFrame)) // frames are always placed
+		                                           // at the end of blocks
+                                                   // so we don't need this 
+	{
+//
+// We have an open hyperlink! FIXME later we should allow this by terminating
+// the hyperlink span just before this strux, then doing the insert strux.
+// Instead for now we'll just disallow this insertStrux.
+//
+// This assert is to remind use to write the code to terminate
+// the hyperlink.
+//
+		pf_Frag * pEndHype = _findNextHyperlink(pf);
+		PT_DocPosition posEnd = 0;
+		if(pEndHype)
+		{
+			posEnd = pEndHype->getPos();
+		}
+		//
+		// OK now insert a new end of hyperlink at pf
+		//
+		insertObject(dpos, PTO_Hyperlink,NULL,NULL);
+		m_fragments.cleanFrags();
+		dpos++;
+		if(posEnd > 0)
+		{
+			//
+			// Now delete the old endhyperlink.
+			//
+			pf_Frag * pfEnd = NULL;
+			UT_uint32 newOff = 0;
+			posEnd++; // from the insert
+			UT_uint32 offset = 0;
+			_deleteObjectWithNotify(posEnd,
+									static_cast<pf_Frag_Object*>(pEndHype),
+									offset,1,pfsContainer,&pfEnd,&newOff,true);
+		}
+		m_fragments.cleanFrags();
+		bFoundFrag = getFragFromPosition(dpos,&pf,&fragOffset);
+		UT_return_val_if_fail (bFoundFrag, false);
+	}	
+
 //
 // If desired, merge in the specified attributes/properties. This
 // enables cells to inherit the properties of the block from which
@@ -386,13 +555,15 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 	{
 		PT_AttrPropIndex pAPIold = indexAP;
 		bool bMerged = m_varset.mergeAP(PTC_AddFmt,pAPIold,attributes,properties,&indexAP,getDocument());
-		UT_ASSERT(bMerged);
+		UT_ASSERT_HARMLESS(bMerged);
 	}
 
 	pf_Frag_Strux * pfsNew = NULL;
 	if (!_createStrux(pts,indexAP,&pfsNew))
 		return false;
 
+	pfsNew->setXID(getXID());
+	
 	// when inserting paragraphs, we try to remember the current
 	// span formatting active at the insertion point and add a
 	// FmtMark immediately after the block.  this way, if the
@@ -450,11 +621,16 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 
 	// create a change record to describe the change, add
 	// it to the history, and let our listeners know about it.
-
+	if(pfsNew->getStruxType() == PTX_SectionFrame)
+	{
+		// Inserting a sectionFrame screws up dos. It goes just before the next
+		// block strux found.
+		dpos = pfsNew->getPrev()->getPos() + pfsNew->getPrev()->getLength();
+	}
 	PX_ChangeRecord_Strux * pcrs
 		= new PX_ChangeRecord_Strux(PX_ChangeRecord::PXT_InsertStrux,
-									dpos,indexAP,pts);
-	UT_ASSERT(pcrs);
+									dpos,indexAP,pfsNew->getXID(), pts);
+	UT_return_val_if_fail (pcrs,false);
 
 	// add record to history.  we do not attempt to coalesce these.
 	m_history.addChangeRecord(pcrs);
@@ -462,7 +638,7 @@ bool pt_PieceTable::_realInsertStrux(PT_DocPosition dpos,
 
 	if (bNeedGlob)
 	{
-		UT_ASSERT(!pfsNew->getNext() || pfsNew->getNext()->getType()!=pf_Frag::PFT_FmtMark);
+		UT_return_val_if_fail (!pfsNew->getNext() || pfsNew->getNext()->getType()!=pf_Frag::PFT_FmtMark, false);
 		_insertFmtMarkAfterBlockWithNotify(pfsNew,dpos+pfsNew->getLength(),apFmtMark);
 		endMultiStepGlob();
 	}
@@ -501,7 +677,7 @@ bool pt_PieceTable::_computeFmtMarkForNewBlock(pf_Frag_Strux * /* pfsNewBlock */
 		{
 		default:
 			{
-				UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+				UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 				return false;
 			}
 
