@@ -53,7 +53,7 @@ static void s_merge_right(GtkWidget *widget, gpointer data )
 	dlg->onMerge();
 }
 
-void s_response(GtkWidget * wid, gint id, AP_UnixDialog_MergeCells * me )
+static void s_response(GtkWidget * wid, gint id, AP_UnixDialog_MergeCells * me )
 {
     abiDestroyWidget( wid ) ;// will emit signals for us
 }
@@ -83,8 +83,7 @@ static void s_merge_below(GtkWidget *widget, gpointer data )
 XAP_Dialog * AP_UnixDialog_MergeCells::static_constructor(XAP_DialogFactory * pFactory,
 													       XAP_Dialog_Id id)
 {
-	AP_UnixDialog_MergeCells * p = new AP_UnixDialog_MergeCells(pFactory,id);
-	return p;
+	return new AP_UnixDialog_MergeCells(pFactory,id);
 }
 
 AP_UnixDialog_MergeCells::AP_UnixDialog_MergeCells(XAP_DialogFactory * pDlgFactory,
@@ -206,7 +205,9 @@ GtkWidget * AP_UnixDialog_MergeCells::_constructWindowContents(void)
 	GtkWidget *wMergeAbove;
 	GtkWidget *wMergeBelow;
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-	frame1 = gtk_frame_new (pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Frame).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Frame,s);
+	frame1 = gtk_frame_new (s.utf8_str());
 	gtk_widget_show (frame1);
 	gtk_container_add (GTK_CONTAINER (wContents), frame1);
 	gtk_container_set_border_width (GTK_CONTAINER (frame1), 3);
@@ -217,27 +218,31 @@ GtkWidget * AP_UnixDialog_MergeCells::_constructWindowContents(void)
 	gtk_container_add (GTK_CONTAINER (frame1), table1);
 	gtk_table_set_col_spacings (GTK_TABLE (table1), 2);
 
-	wlMergeLeft = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Left).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Left,s);
+	wlMergeLeft = gtk_label_new (s.utf8_str());
 	gtk_widget_show (wlMergeLeft);
 	gtk_table_attach (GTK_TABLE (table1), wlMergeLeft, 0, 1, 0, 1,
 					  (GtkAttachOptions) (GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (wlMergeLeft), 0, 0.5);
-	wlMergeRight = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Right).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Right,s);
+	wlMergeRight = gtk_label_new (s.utf8_str());
 	gtk_widget_show (wlMergeRight);
 	gtk_table_attach (GTK_TABLE (table1), wlMergeRight, 0, 1, 1, 2,
 					  (GtkAttachOptions) (GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (wlMergeRight), 0, 0.5);
 
-	wlMergeAbove = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Above).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Above,s);
+	wlMergeAbove = gtk_label_new (s.utf8_str());
 	gtk_widget_show (wlMergeAbove);
 	gtk_table_attach (GTK_TABLE (table1), wlMergeAbove, 0, 1, 2, 3,
 					  (GtkAttachOptions) (GTK_FILL),
 					  (GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (wlMergeAbove), 0, 0.5);
 
-	wlMergeBelow = gtk_label_new (pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Below).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_MergeCells_Below,s);
+	wlMergeBelow = gtk_label_new (s.utf8_str());
 	gtk_widget_show (wlMergeBelow);
 	gtk_table_attach (GTK_TABLE (table1), wlMergeBelow, 0, 1, 3, 4,
                     (GtkAttachOptions) (GTK_FILL),
