@@ -17,6 +17,8 @@
  * 02111-1307, USA.
  */
 
+#include "ap_Features.h"
+
 #include "ut_assert.h"
 #include "ut_vector.h"
 #include "ap_UnixToolbar_SizeCombo.h"
@@ -56,7 +58,7 @@ bool AP_UnixToolbar_SizeCombo::populate(void)
 	    // populate the vector	
 	    int sz = XAP_EncodingManager::fontsizes_mapping.size();
 	    for(int i=0;i<sz;++i) {
-		m_vecContents.addItem(static_cast<const void *>(XAP_EncodingManager::fontsizes_mapping.nth2(i)));
+		m_vecContents.addItem(static_cast<const char *>(XAP_EncodingManager::fontsizes_mapping.nth2(i)));
 	    };
 	}
 	// TODO: may want to populate this based on current font instead?
