@@ -20,6 +20,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+
+#include "ap_Features.h"
+
 #include "ut_assert.h"
 #include "ut_string.h"
 #include "ut_debugmsg.h"
@@ -53,6 +56,7 @@ AP_Dialog_Styles::AP_Dialog_Styles(XAP_DialogFactory * pDlgFactory, XAP_Dialog_I
 	m_pParaPreview = NULL;
 	m_pCharPreview = NULL;
 	m_pAbiPreview = NULL;
+	
 	if(m_vecCharProps.getItemCount() > 0)
 		m_vecCharProps.clear();
 }
@@ -97,8 +101,8 @@ void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp,
 	{
 		char * pSP = UT_strdup(pszProp);
 		char * pSV = UT_strdup(pszVal);
-		m_vecAllProps.addItem((void *) pSP);
-		m_vecAllProps.addItem((void *) pSV);
+		m_vecAllProps.addItem(pSP);
+		m_vecAllProps.addItem(pSV);
 		return;
 	}
 	UT_sint32 i = 0;
@@ -110,17 +114,17 @@ void AP_Dialog_Styles::addOrReplaceVecProp(const XML_Char * pszProp,
 	}
 	if(i < iCount)
 	{
-		char * pSV = (char *) m_vecAllProps.getNthItem(i+1);
+		char * pSV = (char*)m_vecAllProps.getNthItem(i+1);
 		FREEP(pSV);
 		pSV = UT_strdup(pszVal);
-		m_vecAllProps.setNthItem(i+1, (void *) pSV, NULL);
+		m_vecAllProps.setNthItem(i+1, pSV, NULL);
 	}
 	else
 	{
 		char * pSP = UT_strdup(pszProp);
 		char * pSV = UT_strdup(pszVal);
-		m_vecAllProps.addItem((void *) pSP);
-		m_vecAllProps.addItem((void *) pSV);
+		m_vecAllProps.addItem(pSP);
+		m_vecAllProps.addItem(pSV);
 	}
 	return;
 }
@@ -174,8 +178,8 @@ void AP_Dialog_Styles::addOrReplaceVecAttribs(const XML_Char * pszProp,
 	{
 		char * pSP = UT_strdup(pszProp);
 		char * pSV = UT_strdup(pszVal);
-		m_vecAllAttribs.addItem((void *) pSP);
-		m_vecAllAttribs.addItem((void *) pSV);
+		m_vecAllAttribs.addItem(pSP);
+		m_vecAllAttribs.addItem(pSV);
 		return;
 	}
 	UT_sint32 i = 0;
@@ -187,17 +191,17 @@ void AP_Dialog_Styles::addOrReplaceVecAttribs(const XML_Char * pszProp,
 	}
 	if(i < iCount)
 	{
-		char * pSV = (char *) m_vecAllAttribs.getNthItem(i+1);
+		char * pSV = (char*)m_vecAllAttribs.getNthItem(i+1);
 		FREEP(pSV);
 		pSV = UT_strdup(pszVal);
-		m_vecAllAttribs.setNthItem(i+1, (void *) pSV, NULL);
+		m_vecAllAttribs.setNthItem(i+1, pSV, NULL);
 	}
 	else
 	{
 		char * pSP = UT_strdup(pszProp);
 		char * pSV = UT_strdup(pszVal);
-		m_vecAllAttribs.addItem((void *) pSP);
-		m_vecAllAttribs.addItem((void *) pSV);
+		m_vecAllAttribs.addItem(pSP);
+		m_vecAllAttribs.addItem(pSV);
 	}
 	return;
 }
@@ -231,7 +235,10 @@ void AP_Dialog_Styles::fillVecFromCurrentPoint(void)
 	{
 		szName = paraProps[i];
 		szValue = paraProps[i+1];
-		addOrReplaceVecProp(szName,szValue);
+		if(strstr(szName,"toc-") == NULL)
+		{
+			addOrReplaceVecProp(szName,szValue);
+		}
 		i = i + 2;
 	}
 	i = 0;
@@ -423,7 +430,7 @@ void AP_Dialog_Styles::ModifyLang(void)
 
 	XAP_Dialog_Language * pDialog
 		= (XAP_Dialog_Language *)(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 	const XML_Char ** props_in = NULL;
 	if (getView()->getCharFormat(&props_in))
@@ -466,7 +473,7 @@ void AP_Dialog_Styles::ModifyFont(void)
 
 	XAP_Dialog_FontChooser * pDialog
 		= (XAP_Dialog_FontChooser *)(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 	// stuff the GR_Graphics into the dialog so that it
 	// can query the system for font info relative to our
@@ -491,7 +498,7 @@ void AP_Dialog_Styles::ModifyFont(void)
 // Set the background color for the preview
 //
 	static XML_Char  background[8];
-	UT_RGBColor * bgCol = getView()->getCurrentPage()->getOwningSection()->getPaperColor();
+	UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
 	sprintf(background, "%02x%02x%02x",bgCol->m_red,
 			bgCol->m_grn,bgCol->m_blu);
 	pDialog->setBackGroundColor( (const XML_Char *) background);
@@ -639,7 +646,7 @@ s_TabSaveCallBack (AP_Dialog_Tab * pDlg, FV_View * pView,
 				   const char * szTabStops, const char * szDflTabStop,
 				   void * closure)
 {
-	UT_ASSERT(closure);
+	UT_return_if_fail (closure);
 
 	AP_Dialog_Styles * pStyleDlg = static_cast<AP_Dialog_Styles *>(closure);
 
@@ -663,7 +670,7 @@ void AP_Dialog_Styles::ModifyTabs(void)
 
 	AP_Dialog_Tab * pDialog
 		= (AP_Dialog_Tab *)(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 	pDialog->setSaveCallback(s_TabSaveCallBack, (void *)this);
 
@@ -679,7 +686,7 @@ void AP_Dialog_Styles::ModifyTabs(void)
 void AP_Dialog_Styles::ModifyLists(void)
 {
 	UT_DEBUGMSG(("DOM: Doing stuff in Modify Lists \n"));
-	UT_Vector vp;
+	UT_GenericVector<const XML_Char*> vp;
 
 //
 // Fire up a Modal version of Lists dialog
@@ -695,15 +702,15 @@ void AP_Dialog_Styles::ModifyLists(void)
 	AP_Dialog_Lists * pDialog
 		= (AP_Dialog_Lists *)(pDialogFactory->justMakeTheDialog(AP_DIALOG_ID_LISTS));
 
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 //
 // Fill input list for Lists dialog
 //
 	if(getPropsVal("list-style"))
 	{
-		vp.addItem((void *) "list-style");
-		vp.addItem((void *) getPropsVal("list-style"));
+		vp.addItem("list-style");
+		vp.addItem(getPropsVal("list-style"));
 	}
 	if(getPropsVal("field-font"))
 	{
@@ -712,33 +719,33 @@ void AP_Dialog_Styles::ModifyLists(void)
 	}
 	if(getPropsVal("start-value"))
 	{
-		vp.addItem((void *) "start-value");
-		vp.addItem((void *) getPropsVal("start-value"));
+		vp.addItem("start-value");
+		vp.addItem(getPropsVal("start-value"));
 	}
 	if(getPropsVal("list-delim"))
 	{
-		vp.addItem((void *) "list-delim");
-		vp.addItem((void *) getPropsVal("list-delim"));
+		vp.addItem("list-delim");
+		vp.addItem(getPropsVal("list-delim"));
 	}
 	if(getPropsVal("margin-left"))
 	{
-		vp.addItem((void *) "margin-left");
-		vp.addItem((void *) getPropsVal("margin-left"));
+		vp.addItem("margin-left");
+		vp.addItem(getPropsVal("margin-left"));
 	}
 	if(getPropsVal("field-font"))
 	{
-		vp.addItem((void *) "field-font");
-		vp.addItem((void *) getPropsVal("field-font"));
+		vp.addItem("field-font");
+		vp.addItem(getPropsVal("field-font"));
 	}
 	if(getPropsVal("list-decimal"))
 	{
-		vp.addItem((void *) "list-decimal");
-		vp.addItem((void *) getPropsVal("list-decimal"));
+		vp.addItem("list-decimal");
+		vp.addItem(getPropsVal("list-decimal"));
 	}
 	if(getPropsVal("text-indent"))
 	{
-		vp.addItem((void *) "text-indent");
-		vp.addItem((void *) getPropsVal("text-indent"));
+		vp.addItem("text-indent");
+		vp.addItem(getPropsVal("text-indent"));
 	}
 	pDialog->fillDialogFromVector(&vp);
 //
@@ -813,7 +820,7 @@ void AP_Dialog_Styles::ModifyParagraph(void)
 
 	AP_Dialog_Paragraph * pDialog
 		= (AP_Dialog_Paragraph *)(pDialogFactory->requestDialog(AP_DIALOG_ID_PARAGRAPH));
-	UT_ASSERT(pDialog);
+	UT_return_if_fail (pDialog);
 
 	const static XML_Char * paraFields[] = {"text-align", "text-indent", "margin-left", "margin-right", "margin-top", "margin-bottom", "line-height","tabstops","start-value","list-delim", "list-decimal","list-style","field-font","field-color", "keep-together","keep-with-next","orphans","widows","dom-dir"};
 
@@ -861,7 +868,7 @@ void AP_Dialog_Styles::ModifyParagraph(void)
 	{
 		// getDialogData() returns us XML_Char ** data we have to free
 		pDialog->getDialogData(props);
-		UT_ASSERT(props);
+		UT_return_if_fail (props);
 
 		// set properties into the vector. We have to save these as static char
         // strings so they persist past this method.
@@ -934,7 +941,7 @@ void AP_Dialog_Styles::updateCurrentStyle(void)
 		if(i+2<countp)
 			m_curStyleDesc += "; ";
 	}
-	UT_DEBUGMSG(("New props of style %s \n",m_curStyleDesc.c_str()));
+	xxx_UT_DEBUGMSG(("New props of style %s \n",m_curStyleDesc.c_str()));
 //
 // Update the description in the Modify Dialog.
 //
@@ -1025,9 +1032,8 @@ bool AP_Dialog_Styles::createNewStyle(const XML_Char * szName)
 //
 	PD_Style * pStyle = NULL;
 
-	UT_ASSERT(szName);
-	if(szName == NULL)
-		return false;
+	UT_return_val_if_fail (szName, false);
+
 	getDoc()->getStyle("szName", &pStyle);
 	if(pStyle != NULL)
 		return false;
@@ -1036,7 +1042,7 @@ bool AP_Dialog_Styles::createNewStyle(const XML_Char * szName)
 //
 	const XML_Char * attrib[] = {PT_NAME_ATTRIBUTE_NAME,szName,PT_TYPE_ATTRIBUTE_NAME,getAttsVal(PT_TYPE_ATTRIBUTE_NAME),"basedon",getAttsVal("basedon"),"followedby",getAttsVal("followedby"),"props",m_curStyleDesc.c_str(),NULL,NULL};
 	bool bres = getDoc()->appendStyle(attrib);
-	DELETEP(props);
+	FREEP(props);
 	return bres;
 }
 
@@ -1164,16 +1170,17 @@ void AP_Dialog_Styles::_createParaPreviewFromGC(GR_Graphics * gc,
                                                 UT_uint32 width,
 						UT_uint32 height)
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
+	UT_return_if_fail (pSS);
+	
 	UT_UCSChar * str;
 
 	UT_UCS4_cloneString_char (&str, pSS->getValue(AP_STRING_ID_DLG_Styles_LBL_TxtMsg));
 
 	m_pParaPreview = new AP_Preview_Paragraph(gc, str, static_cast<XAP_Dialog*>(this));
-	UT_ASSERT(m_pParaPreview);
+	UT_return_if_fail (m_pParaPreview);
 
 	FREEP(str);
 
@@ -1185,19 +1192,20 @@ void AP_Dialog_Styles::_createCharPreviewFromGC(GR_Graphics * gc,
                                                 UT_uint32 width,
 						UT_uint32 height)
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
+	UT_return_if_fail (pSS);
+	
 //
 // Set the Background color for the preview.
 //
 	static XML_Char  background[8];
-	UT_RGBColor * bgCol = getView()->getCurrentPage()->getOwningSection()->getPaperColor();
+	UT_RGBColor * bgCol = getView()->getCurrentPage()->getFillType()->getColor();
 	sprintf(background, "%02x%02x%02x",bgCol->m_red,bgCol->m_grn,bgCol->m_blu);
 
 	m_pCharPreview = new XAP_Preview_FontPreview(gc,background);
-	UT_ASSERT(m_pCharPreview);
+	UT_return_if_fail (m_pCharPreview);
 
 	m_pCharPreview->setWindowSize(width, height);
 //
@@ -1221,11 +1229,11 @@ void AP_Dialog_Styles::_createAbiPreviewFromGC(GR_Graphics * gc,
                                                 UT_uint32 width,
 											   UT_uint32 height )
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 	if(m_pAbiPreview)
 		DELETEP(m_pAbiPreview);
 	m_pAbiPreview = new AP_Preview_Abi(gc,width,height,getFrame(),PREVIEW_ZOOMED);
-	UT_ASSERT(m_pAbiPreview);
+	UT_return_if_fail (m_pAbiPreview);
 }
 
 /*!
@@ -1328,13 +1336,13 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 				FGColor.m_grn, FGColor.m_blu);
 	if(pszBGColor == NULL)
 	{
-		pageCol = getLView()->getCurrentPage()->getOwningSection()->getPaperColor();
+		pageCol = getLView()->getCurrentPage()->getFillType()->getColor();
 		sprintf(Grey, "%02x%02x%02x",(pageCol->m_red+FGColor.m_red)/2,
 				(pageCol->m_grn+FGColor.m_grn)/2, (pageCol->m_blu+FGColor.m_blu)/2);
 	}
 	else if(strcmp(pszBGColor,"transparent")==0)
 	{
-		pageCol = getLView()->getCurrentPage()->getOwningSection()->getPaperColor();
+		pageCol = getLView()->getCurrentPage()->getFillType()->getColor();
 		sprintf(Grey, "%02x%02x%02x",(pageCol->m_red+FGColor.m_red)/2,
 				(pageCol->m_grn+FGColor.m_grn)/2, (pageCol->m_blu+FGColor.m_blu)/2);
 	}
@@ -1397,7 +1405,7 @@ void AP_Dialog_Styles::_populateAbiPreview(bool isNew)
 // Update the description in the Modify Dialog.
 //
 	setModifyDescription (m_curStyleDesc.c_str());
-	UT_DEBUGMSG(("Style desc is %s \n",m_curStyleDesc.c_str()));
+	xxx_UT_DEBUGMSG(("Style desc is %s \n",m_curStyleDesc.c_str()));
 	if( pStyle == NULL)
 	{
 		if(strlen(m_curStyleDesc.c_str()) == 0)
@@ -1484,7 +1492,7 @@ void AP_Dialog_Styles::event_paraPreviewUpdated (const XML_Char * pageLeftMargin
 	const char * sz = NULL;
 	const char * pPlusFound = NULL;
 
-	UT_ASSERT(m_pParaPreview);
+	UT_return_if_fail (m_pParaPreview);
 
 	if (!align)
 		goto LblIndent; // skip to the next label if nothing's set here
@@ -1554,14 +1562,11 @@ void AP_Dialog_Styles::event_paraPreviewUpdated (const XML_Char * pageLeftMargin
  */
 void AP_Dialog_Styles::event_charPreviewUpdated (void) const
 {
-	UT_ASSERT (m_pCharPreview); // add this when we make a char preview
+	UT_return_if_fail (m_pCharPreview); // add this when we make a char preview
 
 	// force a redraw
-	if(m_pCharPreview)
-	{
-		m_pCharPreview->setVecProperties( &m_vecCharProps);
-		m_pCharPreview->draw();
-	}
+	m_pCharPreview->setVecProperties( &m_vecCharProps);
+	m_pCharPreview->draw();
 }
 
 /*!
