@@ -968,6 +968,8 @@ dcl(MENU_LABEL_FMT_IMAGE, "&Image")
 dcl(MENU_LABEL_FMT_SETPOSIMAGE, "Set Positioned Image")
 dcl(MENU_LABEL_FMT_POSIMAGE, "&Image")
 dcl(MENU_LABEL_FMT_DIRECTION, "D&irectional")
+dcl(MENU_LABEL_FMT_DIRECTION_DOCD_RTL, "RTL &Document")
+dcl(MENU_LABEL_FMT_DIRECTION_SD_RTL, "RTL &Section")
 dcl(MENU_LABEL_FMT_DIRECTION_DD_RTL, "RTL &Paragraph")
 dcl(MENU_LABEL_FMT_DIRECTION_DO_LTR, "&Left-to-right text")
 dcl(MENU_LABEL_FMT_DIRECTION_DO_RTL, "&Right-to-left text")
@@ -1238,6 +1240,8 @@ dcl(MENU_STATUSLINE_FMT_SUPERSCRIPT,"Make the selection superscript (toggle)")
 dcl(MENU_STATUSLINE_FMT_SUBSCRIPT,"Make the selection subscript (toggle)")
 dcl(MENU_STATUSLINE_FMT_FRAME,"Change the properties of the Text Box")
 dcl(MENU_STATUSLINE_FMT_DIRECTION, "Change directional properties of text")
+dcl(MENU_STATUSLINE_FMT_DIRECTION_DOCD_RTL, "Set dominant direction of document to RTL")
+dcl(MENU_STATUSLINE_FMT_DIRECTION_SD_RTL, "Set dominant direction of section to RTL")
 dcl(MENU_STATUSLINE_FMT_DIRECTION_DD_RTL, "Set dominant direction of paragraph to RTL")
 dcl(MENU_STATUSLINE_FMT_DIRECTION_DO_LTR, "Force left-to-right direction of text")
 dcl(MENU_STATUSLINE_FMT_DIRECTION_DO_RTL, "Force right-to-left direction of text")
