@@ -376,7 +376,7 @@ static bool AbiGOChart_Create(AV_View* v, EV_EditMethodCallData *d);
 // FIXME make these translatable strings
 
 static const char * Object_MenuLabelObject = "Object";
-static const char * Object_MenuTooltipObject = "Insert Emebable Object";
+static const char * Object_MenuTooltipObject = "Insert Embeddable Object";
 static const char* AbiGOChart_MenuLabelInsert = "Gnome Office Chart";
 static const char* AbiGOChart_MenuTooltipInsert = "Create a chart";
 
@@ -932,7 +932,8 @@ void GR_GOChartManager::makeSnapShot(UT_sint32 uid, UT_Rect & rec)
   UT_return_if_fail(pItem);  
   const PP_AttrProp * pSpanAP = NULL;
   PT_AttrPropIndex api = pItem->m_iAPI;
-  /* bool b = */ m_pDoc->getAttrProp(api, &pSpanAP);
+  bool bHaveProp = m_pDoc->getAttrProp(api, &pSpanAP);
+  UT_return_if_fail(bHaveProp);
   const char * pszDataID = NULL;
   bool bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
   if(pItem->m_bHasSnapshot)
@@ -963,8 +964,16 @@ bool GR_GOChartManager::createPNGSnapshot(AD_Document * pDoc, UT_Rect & rec,
   {
     return false;
   }
+  if((rec.width == 0) || (rec.height ==0))
+  {
+    return false;
+  }
   GR_Painter painter(getGraphics());
   GR_Image * pImage = painter.genImageFromRectangle(rec);
+  if(pImage == NULL)
+  {
+    return false;
+  }
   UT_ByteBuf * pBuf = NULL;
   pImage->convertToBuffer(&pBuf);
   UT_UTF8String sID = "snapshot-png-";
@@ -987,6 +996,10 @@ bool GR_GOChartManager::updatePNGSnapshot(AD_Document * pDoc, UT_Rect & rec,
   }
   GR_Painter painter(getGraphics());
   GR_Image * pImage = painter.genImageFromRectangle(rec);
+  if(pImage == NULL)
+  {
+    return false;
+  }
   UT_ByteBuf * pBuf = NULL;
   pImage->convertToBuffer(&pBuf);
   UT_UTF8String sID = "snapshot-png-";
@@ -1022,7 +1035,8 @@ void GR_GOChartManager::loadEmbedData(UT_sint32 uid)
   GR_AbiGOChartItems * pItem = m_vecItems.getNthItem(uid);
   UT_return_if_fail(pItem);  
   PT_AttrPropIndex api = pItem->m_iAPI;
-  /* bool b = */ m_pDoc->getAttrProp(api, &pSpanAP);
+  bool bHaveProp = m_pDoc->getAttrProp(api, &pSpanAP);
+  UT_return_if_fail(bHaveProp);
   const char * pszDataID = NULL;
   bool bFoundDataID = pSpanAP->getAttribute("dataid", pszDataID);
   UT_UTF8String sGOChartXML;
@@ -1033,8 +1047,11 @@ void GR_GOChartManager::loadEmbedData(UT_sint32 uid)
 						    const_cast<const UT_ByteBuf **>(&pByteBuf),
 						    NULL, NULL);
 
-       UT_UCS4_mbtowc myWC;
-       sGOChartXML.appendBuf( *pByteBuf, myWC);
+       if (bFoundDataID)
+       {
+            UT_UCS4_mbtowc myWC;
+            sGOChartXML.appendBuf( *pByteBuf, myWC);
+       }
   }
  UT_return_if_fail(bFoundDataID);
  UT_return_if_fail(pszDataID);
@@ -1103,6 +1120,7 @@ GOChartView::GOChartView(GR_GOChartManager * pGOMan): m_pGOMan(pGOMan)
 	m_Graph = NULL;
 	m_Image = NULL;
 	m_Renderer = GOG_RENDERER (g_object_new (GOG_RENDERER_PIXBUF_TYPE, NULL));
+	pix_width = pix_height = 0;
 	width = height = 0;
 	m_Guru = NULL;
 }
@@ -1120,14 +1138,19 @@ GOChartView::~GOChartView(void)
 
 void GOChartView::render(UT_Rect & rec)
 {
+	if((rec.width == 0) || (rec.height ==0))
+	{
+		return;
+	}
 	if (m_pGOMan->getGraphics()->queryProperties(GR_Graphics::DGP_PAPER))
 	{
-	        XAP_UnixGnomePrintGraphics * pUGG = static_cast< XAP_UnixGnomePrintGraphics *>(m_pGOMan->getGraphics());
+	    XAP_UnixGnomePrintGraphics * pUGG = static_cast< XAP_UnixGnomePrintGraphics *>(m_pGOMan->getGraphics());
 		GnomePrintContext *ctx = pUGG->getGnomePrintContext();
 		UT_sint32 myWidth = pUGG->tdu(rec.width);
 		UT_sint32 myHeight = pUGG->tdu(rec.height);
 		UT_sint32 x = pUGG->tdu(rec.left);
 		UT_sint32 y = pUGG->tdu(rec.top);
+		
 		//
 		// gnome_print places y = 0 at the bottom of the page
 		// and as y increases, it gets placed further from the bottom.
@@ -1152,11 +1175,20 @@ void GOChartView::render(UT_Rect & rec)
 		UT_sint32 _height = pUGG->tdu(rec.height);
 		UT_sint32 x = pUGG->tdu(rec.left);
 		UT_sint32 y = pUGG->tdu(rec.top);
-		if (_width != width || _height != height)
+		UT_sint32 zoom = pUGG->getZoomPercentage ();
+		UT_sint32 real_width = _width * 100 / zoom;
+		UT_sint32 real_height = _height * 100 / zoom;
+		if (real_width != width || real_height != height)
+		{
+			width = real_width;
+			height = real_height;
+			gog_graph_set_size (m_Graph, width, height);
+		}
+		if (_width != pix_width || _height != pix_height)
 		{
 			GdkPixbuf *pixbuf;
-			width = _width;
-			height = _height;
+			pix_width = _width;
+			pix_height = _height;
 			gog_renderer_pixbuf_update (GOG_RENDERER_PIXBUF (m_Renderer), _width, _height, 1.0);	
 			if (m_Image)
 				delete m_Image;
@@ -1253,10 +1285,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
  	GO_DATA_SCALAR_VAL_TYPE;
 	GO_DATA_VECTOR_VAL_TYPE;
  	GO_DATA_MATRIX_VAL_TYPE;
-	/* the resolution of symbols in libgoffice must be forced because when
-	creating the guru window, libglade needs foo_canvas_new. There might
-	be a better solution..., I could not find one */
-	g_module_close (g_module_open ("libgoffice-1", (GModuleFlags)0));
    return 1;
 }
 
