@@ -105,8 +105,9 @@ private:
 	GogGraph *m_Graph;
 	GogRenderer *m_Renderer;
 	GR_Image *m_Image;
-	UT_sint32 width;
-	UT_sint32 height;
+	UT_sint32 width, height;
+	UT_sint32 pix_width;
+	UT_sint32 pix_height;
 	fp_Run *m_pRun;
 	GtkWidget *m_Guru;
 };
