@@ -99,7 +99,7 @@ const static AbiMenuOptions amo [] =
 #define NUM_MENUITEMS NrElements(amo)
 
 const char * prevMM = "&Word Count";	// Main menu item we place our 1st menu item after
-const char * prevCM = "Resize &Image";	// Context menu item we place our 1st image context menu item after
+const char * prevCM = "Format &Image";	// Context menu item we place our 1st image context menu item after
 
 
     
@@ -113,7 +113,7 @@ const char * prevCM = "Resize &Image";	// Context menu item we place our 1st ima
 static XAP_ModuleInfo AbiPaintModuleInfo = 
 {
     "AbiPaint",											/* name */
-    "Allows editting an embedded image via external image editing program.",	/* desc */
+    "Allows editing an embedded image via external image editing program.",	/* desc */
     ABI_PLUGIN_mkstr(ABI_PLUGIN_VERSION) " for AbiWord " ABI_BUILD_VERSION , 	/* version */
     "Abi the Ant",										/* author */
     "Select Image 1st, then select the action from AbiPaint menu.  ;-)",	/* usage */
@@ -538,7 +538,7 @@ static DECLARE_ABI_PLUGIN_METHOD(editImage)
 				pView->cmdUnselectSelection();
 				pView->setPoint(pos);
 				pView->extSelHorizontal(true, 1); // move point forward one
-				errorCode = pView->cmdInsertGraphic(pFG, szTmpPng.c_str());
+				errorCode = pView->cmdInsertGraphic(pFG);
 				if (errorCode)
 				{
 					pFrame->showMessageBox("Could not put image back into Abiword", XAP_Dialog_MessageBox::b_O,XAP_Dialog_MessageBox::a_OK);
