@@ -32,7 +32,7 @@
 #include <ut_math.h>
 #include <fp_PageSize.h>
 #include <pd_Document.h>
-
+#include <ut_locale.h>
 
 /**
  * 
@@ -57,6 +57,9 @@ void ODi_Style_PageLayout::startElement(const XML_Char* pName,
         
         pVal = UT_getAttribute ("fo:column-count", ppAtts);
         if (pVal) {
+            if (!UT_strcmp(pVal,"0")) {  // see Bug 10076
+                pVal = "1";
+            }
             m_columnCount = pVal;
         }
         
@@ -96,6 +99,7 @@ void ODi_Style_PageLayout::definePageSizeTag(PD_Document* pDocument) const {
     double pageHeightMmNumeric;
     UT_UTF8String pageWidthMm;
     UT_UTF8String pageHeightMm;
+    UT_LocaleTransactor lt(LC_NUMERIC, "C");
     
     // width and height are rounded to full mm because that's how they are
     // predefined in Abi and there seem to be rounding errors in oow's exporter
@@ -217,6 +221,7 @@ void ODi_Style_PageLayout::_parsePageLayoutProperties(const XML_Char** ppAtts) {
 void ODi_Style_PageLayout::_buildSectionPropsString() {
     double val;
     UT_UTF8String str;
+    UT_LocaleTransactor lt(LC_NUMERIC, "C");
     
     m_sectionProps.clear();
         
