@@ -526,7 +526,7 @@ void OO_Listener::_openBlock (PT_AttrPropIndex api)
 
 	if (bHaveProp && pAP)
 	{
-		UT_UTF8String sa, pa, f;
+		UT_UTF8String sa, pa, f, escape;
 		OO_StylesWriter::map(pAP, sa, pa, f);
 
 		const XML_Char * szStyle = NULL;
@@ -535,10 +535,14 @@ void OO_Listener::_openBlock (PT_AttrPropIndex api)
 		if (szStyle && pa.size())
 		{
 			// custom properties, prepare to derive style
-			sa += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", szStyle);
+			escape = szStyle;
+			sa += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", escape.escapeXML().utf8_str());
 		}
 		else if (szStyle)
-			sa += UT_UTF8String_sprintf("text:style-name=\"%s\" ", szStyle);
+		{
+			escape = szStyle;
+			sa += UT_UTF8String_sprintf("text:style-name=\"%s\" ", escape.escapeXML().utf8_str());
+		}
 	
 		// FIXME: hacky, but in sxw there is a distinction between heading- and text paragraphs
 		// we should probably also care about text:level here
@@ -637,7 +641,7 @@ public:
       oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
     if (pDoc->getMetaDataProp(PD_META_KEY_LANGUAGE, meta_val) && meta_val.size()) {
-      val = UT_UTF8String_sprintf("<dc:language>%s</dc:language>\n", meta_val.utf8_str());
+      val = UT_UTF8String_sprintf("<dc:language>%s</dc:language>\n", meta_val.escapeXML().utf8_str());
       oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
 
@@ -917,13 +921,17 @@ void OO_StylesWriter::addFontDecls(UT_UTF8String & buffer, OO_StylesContainer &
 
 void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT_UTF8String & propAtts, UT_UTF8String & font) 
 {		
+	UT_UTF8String escape;
 	const XML_Char * szValue = NULL;
 	styleAtts.clear();
 	propAtts.clear();
 
 	// Attributes
-	if (pAP->getAttribute("name", szValue)) 
-		styleAtts += UT_UTF8String_sprintf("style:name=\"%s\" ", szValue);
+	if (pAP->getAttribute("name", szValue))
+	{ 
+		escape = szValue;
+		styleAtts += UT_UTF8String_sprintf("style:name=\"%s\" ", escape.escapeXML().utf8_str());
+	}
 
 	if (pAP->getAttribute("type", szValue)) 
 		if (!UT_strcmp(szValue, "P"))
@@ -933,13 +941,19 @@ void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT
 		}
 
 	if (pAP->getAttribute("basedon", szValue)) 
-		styleAtts += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", szValue);
+	{
+		escape = szValue;
+		styleAtts += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", escape.escapeXML().utf8_str());
+	}
 	
 	if (pAP->getAttribute("followedby", szValue)) 
 	{
 		// ignore, current style is default
 		if (UT_strcmp(szValue, "Current Settings"))
-			styleAtts += UT_UTF8String_sprintf("style:next-style-name=\"%s\" ", szValue);
+		{
+			escape = szValue;
+			styleAtts += UT_UTF8String_sprintf("style:next-style-name=\"%s\" ", escape.escapeXML().utf8_str());
+		}
 	}
 
 	// Properties
