@@ -36,7 +36,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <locale.h>
+#include "ut_locale.h"
 
 #include "ut_types.h"
 #include "ut_assert.h"
@@ -361,7 +361,7 @@ int IE_Imp_MSWrite::read_pap ()
 			if (header) {
 				UT_DEBUGMSG(("Headers and footers not supported, skipping...\n")); 
             } else {
-				setlocale (LC_NUMERIC, "C");
+				UT_LocaleTransactor lt (LC_NUMERIC, "C");
 				UT_String_sprintf (propBuffer, "text-align:%s; line-height:%.1f",
 						   text_align[jc], static_cast<float>(dyaLine) / 240.0);
 				propBuffer += tempBuffer;
@@ -395,7 +395,6 @@ int IE_Imp_MSWrite::read_pap ()
 							 static_cast<float>(dxaRight) / 1440.0);
 					propBuffer += tempBuffer;
 				}
-				setlocale (LC_NUMERIC, "");
 
 				// end of formatting
 
@@ -481,7 +480,7 @@ int IE_Imp_MSWrite::read_char (int fcFirst2, int fcLim2) {
 			
 			if ((fcLim >= fcFirst2) && (fcFirst <= fcLim2))  {
 				mCharBuf.clear ();
-				setlocale (LC_NUMERIC, "C");
+				UT_LocaleTransactor lt (LC_NUMERIC, "C");
 				
 				UT_String_sprintf (propBuffer, "font-weight:%s", bold ? "bold" : "normal");
 				if (italic)  {
@@ -508,7 +507,6 @@ int IE_Imp_MSWrite::read_char (int fcFirst2, int fcLim2) {
 					translate_char (*(mTextBuf.getPointer(fcFirst2 - 0x80)), mCharBuf);
 					fcFirst2++;
 				}
-				setlocale (LC_NUMERIC, "");
 				
 				const XML_Char* propsArray[3];
 				propsArray[0] = pProps;
