@@ -610,9 +610,8 @@ void WordPerfect_Listener::_openSpan(PT_AttrPropIndex api)
 			
 			if (pszFontSize)
 			{
-				setlocale (LC_NUMERIC, "C");
+				UT_LocaleTransactor lt (LC_NUMERIC, "C");
 				_handleFontSizeChange(UT_convertToPoints(pszFontSize));
-				setlocale (LC_NUMERIC, "");
 			}
 			
 			if (pszFontFamily)
