@@ -28,7 +28,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <locale.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-input-stdio.h>
@@ -43,6 +42,7 @@
 #include "ut_debugmsg.h"
 #include "ut_math.h" // for rint (font size)
 #include "ut_rand.h"
+#include "ut_locale.h"
 
 #include "xap_Frame.h"
 #include "xap_EncodingManager.h"
@@ -69,6 +69,8 @@ ABI_ListDefinition::ABI_ListDefinition(int iOutlineHash) :
 		m_iListIDs[i] = 0;
 		m_listTypes[i] = BULLETED_LIST;
 		m_iListNumbers[i] = 0;
+		m_listLeftOffset[i] = 0.0f;
+		m_listMinLabelWidth[i] = 0.0f;
 	}
 }
 
@@ -168,8 +170,11 @@ IE_Imp_WordPerfect::IE_Imp_WordPerfect(PD_Document * pDocument)
 	m_bParagraphInSection(false),
 	m_bInSection(false),
 	m_bSectionChanged(false),
-	m_leftMargin(0.0f),
-	m_rightMargin(0.0f),
+	m_leftPageMargin(1.0f),
+	m_rightPageMargin(1.0f),
+	m_leftSectionMargin(0.0f),
+	m_rightSectionMargin(0.0f),
+	m_sectionColumnsCount(0),
 	m_headerId(-1),
 	m_footerId(-1),
 	m_nextFreeId(0),
@@ -189,8 +194,6 @@ IE_Imp_WordPerfect::~IE_Imp_WordPerfect()
 
 UT_Error IE_Imp_WordPerfect::importFile(const char * szFilename)
 {
-	gsf_init ();
-
 	GError *err;
 	GsfInput * input;
 	input = GSF_INPUT(gsf_input_stdio_new (szFilename, &err));
@@ -256,26 +259,20 @@ void IE_Imp_WordPerfect::openPageSpan(const WPXPropertyList &propList)
 	if (m_bHdrFtrOpenCount) return; // HACK
 	UT_DEBUGMSG(("AbiWordPerfect: openPageSpan\n"));
 	
-	float marginLeft = 0.0f, marginRight = 0.0f;
+	float marginLeft = 1.0f, marginRight = 1.0f;
 
 	if (propList["fo:margin-left"])
 		marginLeft = propList["fo:margin-left"]->getFloat();
 	if (propList["fo:margin-right"])
 		marginRight = propList["fo:margin-right"]->getFloat();
 
-	if (
-		marginLeft != m_leftMargin ||
-		marginRight != m_rightMargin //||
-		//marginTop != m_marginBottom ""
-		//marginBottom != m_marginBottom
-		)
-	{
-		m_leftMargin = marginLeft;
-		m_rightMargin = marginRight;
+	if (marginLeft != m_leftPageMargin || marginRight != m_rightPageMargin /* || */
+		/* marginTop != m_marginBottom || marginBottom != m_marginBottom */ )
+		m_bSectionChanged = true; // margin properties are section properties in AbiWord
+
+	m_leftPageMargin = marginLeft;
+	m_rightPageMargin = marginRight;
 		
-		// margin properties are section properties in AbiWord
-		m_bSectionChanged;
-	}
 }
 
 void IE_Imp_WordPerfect::openHeader(const WPXPropertyList &propList)
@@ -377,6 +374,48 @@ void IE_Imp_WordPerfect::openParagraph(const WPXPropertyList &propList, const WP
 	UT_String_sprintf(tmpBuffer, "; margin-top:%.4fin; margin-bottom:%.4fin; margin-left:%.4fin; margin-right:%.4fin; text-indent:%.4fin; line-height:%.4f",
 		m_topMargin, m_bottomMargin, m_leftMarginOffset, m_rightMarginOffset, m_textIndent, lineSpacing);
 	propBuffer += tmpBuffer;
+	
+	if (tabStops.count() > 0) // Append the tabstop information
+	{
+		propBuffer += "; tabstops:";
+		UT_String_sprintf(tmpBuffer, "");
+                WPXPropertyListVector::Iter i(tabStops);
+                for (i.rewind(); i.next();)
+                {
+			propBuffer += tmpBuffer;
+			if (i()["style:position"])
+			{
+				UT_String_sprintf(tmpBuffer, "%.4fin", i()["style:position"]->getFloat());
+				propBuffer += tmpBuffer;
+			}
+
+			if (i()["style:type"])
+				if (i()["style:type"]->getStr() == "right")
+					propBuffer += "/R";
+				else if (i()["style:type"]->getStr() == "center")
+					propBuffer += "/C";
+				else if (i()["style:type"]->getStr() == "char")
+					propBuffer += "/D";
+				else
+					propBuffer += "/L";
+			else // Left aligned is default
+				propBuffer += "/L";
+
+			if (i()["style:leader-char"])
+				if (i()["style:leader-char"]->getStr() == "-")
+					propBuffer += "2";
+				else if (i()["style:leader-char"]->getStr() == "_")
+					propBuffer += "3";
+				else // default to dot leader if the given leader is dot or is not supported by AbiWord
+					propBuffer += "1";
+			else
+				propBuffer += "0";
+
+			UT_String_sprintf(tmpBuffer, ",");
+                }
+        }
+
+	
 
 	UT_DEBUGMSG(("AbiWordPerfect: Appending paragraph properties: %s\n", propBuffer.c_str()));
 	const XML_Char* propsArray[3];
@@ -476,13 +515,23 @@ void IE_Imp_WordPerfect::openSection(const WPXPropertyList &propList, const WPXP
 	if (m_bHdrFtrOpenCount) return; // HACK
 	UT_DEBUGMSG(("AbiWordPerfect: openSection\n"));
 
+	float marginLeft = 0.0f, marginRight = 0.0f;
+	int columnsCount = ((columns.count() == 0) ? 1 : columns.count());
+
 	// TODO: support spaceAfter
 	if (propList["fo:margin-left"])
-		m_leftMargin = propList["fo:margin-left"]->getFloat();
+		marginLeft = propList["fo:margin-left"]->getFloat();
 	if (propList["fo:margin-right"])
-		m_rightMargin = propList["fo:margin-right"]->getFloat();
+		marginRight = propList["fo:margin-right"]->getFloat();
 
-	_appendSection((columns.count() == 0) ? 1 : columns.count(), m_leftMargin, m_rightMargin); 
+	if (marginLeft != m_leftSectionMargin || marginRight != m_rightSectionMargin || m_sectionColumnsCount != columnsCount)
+		m_bSectionChanged = true;
+
+	m_leftSectionMargin = marginLeft;
+	m_rightSectionMargin = marginRight;
+	m_sectionColumnsCount = columnsCount;
+	
+	_appendSection(columnsCount, m_leftPageMargin + m_leftSectionMargin, m_rightPageMargin + m_rightSectionMargin); 
 }
 
 void IE_Imp_WordPerfect::insertTab()
@@ -524,6 +573,8 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 	int listID = 0, startingNumber = 0, level = 1;
 	char listType = '1';
 	UT_UTF8String textBeforeNumber, textAfterNumber;
+	float listLeftOffset = 0.0f;
+	float listMinLabelWidth = 0.0f;
 	
 	if (propList["libwpd:id"])
 		listID = propList["libwpd:id"]->getInt();
@@ -536,10 +587,12 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 	if (propList["style:num-suffix"])
 		textAfterNumber += propList["style:num-suffix"]->getStr().cstr();
 	if (propList["style:num-format"])
-	{
 		listType = propList["style:num-format"]->getStr().cstr()[0];
-		fprintf(stderr, "About to die: %c\n", propList["style:num-format"]->getStr().cstr()[0]);
-	}
+	if (propList["text:space-before"])
+		listLeftOffset = propList["text:space-before"]->getFloat();
+	if (propList["text:min-label-width"])
+		listMinLabelWidth = propList["text:min-label-width"]->getFloat();
+
 	if (!m_pCurrentListDefinition || 
 		m_pCurrentListDefinition->getOutlineHash() != listID ||
 		(m_pCurrentListDefinition->getLevelNumber(level) != startingNumber && 
@@ -548,7 +601,6 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 		if (m_pCurrentListDefinition)
 			delete (m_pCurrentListDefinition);
 
-		fprintf(stderr, "WLACH: Outline hash - listID: %i\n", listID);
 		m_pCurrentListDefinition = new ABI_ListDefinition(listID);
 	}
 	
@@ -556,6 +608,8 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 	{
 		m_pCurrentListDefinition->setListType(level, listType);
 		m_pCurrentListDefinition->setListID(level, UT_rand());
+		m_pCurrentListDefinition->setListLeftOffset(level, listLeftOffset);
+		m_pCurrentListDefinition->setListMinLabelWidth(level, listMinLabelWidth);
 		_updateDocumentOrderedListDefinition(m_pCurrentListDefinition, level, listType, textBeforeNumber, textAfterNumber, startingNumber);
 	}
 }
@@ -567,10 +621,17 @@ void IE_Imp_WordPerfect::defineUnorderedListLevel(const WPXPropertyList &propLis
 
 	int listID = 0, level = 1;
 	WPXString textBeforeNumber, textAfterNumber;
+	float listLeftOffset = 0.0f;
+	float listMinLabelWidth = 0.0f;
+	
 	if (propList["libwpd:id"])
 		listID = propList["libwpd:id"]->getInt();
 	if (propList["libwpd:level"])
 		level = propList["libwpd:level"]->getInt();
+	if (propList["text:space-before"])
+		listLeftOffset = propList["text:space-before"]->getFloat();
+	if (propList["text:min-label-width"])
+		listMinLabelWidth = propList["text:min-label-width"]->getFloat();
 
 	if (!m_pCurrentListDefinition || m_pCurrentListDefinition->getOutlineHash() != listID)
 	{
@@ -583,6 +644,8 @@ void IE_Imp_WordPerfect::defineUnorderedListLevel(const WPXPropertyList &propLis
 	if (!m_pCurrentListDefinition->getListID(level))
 	{
 		m_pCurrentListDefinition->setListID(level, UT_rand());
+		m_pCurrentListDefinition->setListLeftOffset(level, listLeftOffset);
+		m_pCurrentListDefinition->setListMinLabelWidth(level, listMinLabelWidth);
 		_updateDocumentUnorderedListDefinition(m_pCurrentListDefinition, level);
 	}
 }
@@ -685,16 +748,15 @@ void IE_Imp_WordPerfect::openListElement(const WPXPropertyList &propList, const
 	propBuffer += tempBuffer;
 	UT_String_sprintf(tempBuffer, "start-value:%i; ", 1);
 	propBuffer += tempBuffer;
-	if (propList["fo:text-indent"])
-	{
-		UT_String_sprintf(tempBuffer, "text-indent:%s; ", propList["fo:text-indent"]->getStr().cstr());
-		propBuffer += tempBuffer;
-	}
-	if (propList["fo:margin-left"])
-	{
-		UT_String_sprintf(tempBuffer, "margin-left:%s", propList["fo:margin-left"]->getStr().cstr());
-		propBuffer += tempBuffer;
-	}
+
+	UT_String_sprintf(tempBuffer, "margin-left:%.4fin; ", m_pCurrentListDefinition->getListLeftOffset(m_iCurrentListLevel)
+					+ m_pCurrentListDefinition->getListMinLabelWidth(m_iCurrentListLevel)
+					- (propList["fo:text-indent"] ? propList["fo:text-indent"]->getFloat() : 0.0f));
+	propBuffer += tempBuffer;
+	UT_String_sprintf(tempBuffer, "text-indent:%.4fin", - m_pCurrentListDefinition->getListMinLabelWidth(m_iCurrentListLevel)
+					+ (propList["fo:text-indent"] ? propList["fo:text-indent"]->getFloat() : 0.0f));
+	propBuffer += tempBuffer;
+
 	listAttribs[attribsCount++] = PT_PROPS_ATTRIBUTE_NAME;
 	listAttribs[attribsCount++] = propBuffer.c_str();
 	listAttribs[attribsCount++] = NULL;
@@ -874,7 +936,8 @@ void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
 	}
 	
 	UT_String propBuffer;
-	UT_String_sprintf(propBuffer, "left-attach:%d; right-attach:%d; top-attach:%d; bot-attach:%d", col, col+colSpan, row, row+rowSpan);
+	UT_String_sprintf(propBuffer, "left-attach:%d; right-attach:%d; top-attach:%d; bot-attach:%d",
+					col, col+colSpan, row, row+rowSpan);
 	
 	UT_String borderStyle;
 	// we only support bg-style:1 for now
@@ -940,9 +1003,8 @@ UT_Error IE_Imp_WordPerfect::_appendSection(int numColumns, const float marginLe
 	UT_DEBUGMSG(("AbiWordPerfect: Appending section\n"));
 	
 	UT_String myProps("") ;
-	setlocale(LC_NUMERIC, "C");
+	UT_LocaleTransactor lt(LC_NUMERIC, "C");
 	myProps += UT_String_sprintf("columns:%d; page-margin-left:%.4fin; page-margin-right:%.4fin", numColumns, marginLeft, marginRight);
-	setlocale(LC_NUMERIC, NULL);
 	
 	XML_Char * propsArray[3];
 	propsArray[0] = "props";
@@ -951,10 +1013,7 @@ UT_Error IE_Imp_WordPerfect::_appendSection(int numColumns, const float marginLe
 	X_CheckDocumentError(appendStrux(PTX_Section, (const XML_Char**)propsArray));
 	
 	m_bInSection = true;
-	
-	m_leftMargin = marginLeft;
-	m_rightMargin = marginRight;
-	
+
 	m_bSectionChanged = false;
 	
 	return UT_OK;
@@ -971,7 +1030,7 @@ UT_Error IE_Imp_WordPerfect::_updateDocumentOrderedListDefinition(ABI_ListDefini
 	UT_DEBUGMSG(("AbiWordPerfect: Updating document list definition (iLevel: %i)\n", iLevel));
 
 	if (iLevel > 1)
-	fprintf(stderr, "WLACH: Parent's list id is.. %i\n", pListDefinition->getListID((iLevel-1)));
+	UT_DEBUGMSG(("WLACH: Parent's list id is.. %i\n", pListDefinition->getListID((iLevel-1))));
 
 	// finally, set the document's list identification info..
 	fl_AutoNum * pAuto = getDoc()->getListByID(pListDefinition->getListID(iLevel));
