@@ -1074,7 +1074,10 @@ wvApplysprmPIstdPermute (PAP * apap, U8 * pointer, U16 * pos)
 	    }
       }
     else
-	rgistd = NULL;
+      {
+      rgistd = NULL;
+      return;
+      }
     /*
        First check if pap.istd is greater than the istdFirst recorded in the sprm
        and less than or equal to the istdLast recorded in the sprm If not, the sprm
@@ -1550,7 +1553,7 @@ wvApplysprmPHugePapx (PAP * apap, U8 * pointer, U16 * pos, wvStream * data,
 	  wvError (("No data stream!!\n"));
 	  return;
       }
-    if (0 != wvStream_goto (data, offset))
+    if (0 > wvStream_goto (data, offset))
       {
 	  wvError (("Couldn't seek data stream!!\n"));
 	  apap->fTtp++;
@@ -1659,7 +1662,8 @@ wvApplysprmCIstdPermute (CHP * achp, U8 * pointer, U16 * pos)
     (*pos) += 2;
     istdLast = dread_16ubit (NULL, &pointer);
     (*pos) += 2;
-    if ((cch - 6) / 2 != 0)
+
+    if (cch > 6)
       {
 	  rgistd = (U16 *) wvMalloc (sizeof (U16) * ((cch - 6) / 2));
 	  for (i = 0; i < (cch - 6) / 2; i++)
