@@ -25,9 +25,9 @@ wvnLocaleToLIDConverter (U8 nLocale)
     switch (nLocale)
       {
 #if 0
-	  // case 0:		/* ANSI_CHARSET */
-	  // case 1:		/* DEFAULT_CHARSET */
-	  // case 2:		/* SYMBOL_CHARSET */
+	/* case 0: */		/* ANSI_CHARSET */
+	/* case 1: */		/* DEFAULT_CHARSET */
+	/* case 2: */		/* SYMBOL_CHARSET */
 #endif
 	  case 77:			/* MAC_CHARSET */
 	  return (0xFFF);	/* This number is a hack */
@@ -61,7 +61,7 @@ wvnLocaleToLIDConverter (U8 nLocale)
 	  return (0x405);	/* Czech - And many others!! */
 
 #if 0
-	  // case 255:		/* OEM_CHARSET */
+	  /* case 255: */		/* OEM_CHARSET */
 #endif
 
       default:
@@ -164,417 +164,225 @@ wvOutputHtmlChar (U16 eachchar, U8 chartype, char *outputtype, U16 lid)
        return cpname;  \
 }
 
-#if 0
-
-// todo: use this to cut down on duplicate data
-
 typedef struct {
-  const char * locale;
-  U16 cp;
-  U16 lid;
-} wvLanguageId;
+	const char * language_tag ;
+	U16 lid ;
+} wvLanguageId ;
 
-#endif
+static const wvLanguageId mLanguageIds[] =
+	{
+		{ "-none-", 0x0000 }, /* none (language neutral) */
+		{ "-none-", 0x0400 }, /* none */
+		{ "af-ZA", 0x0436 }, /* Afrikaans */
+		{ "am", 0x045e }, /* Amharic */
+		{ "sq-AL", 0x041c }, /* Albanian */
+		{ "ar-SA", 0x0401 }, /* Arabic (Saudi) */
+		{ "ar-IQ", 0x0801 }, /* Arabic (Iraq) */
+		{ "ar-EG", 0x0c01 }, /* Arabic (Egypt) */		
+		{ "ar-LY", 0x1001 }, /* Arabic (Libya) */
+		{ "ar-DZ", 0x1401 }, /* Arabic (Algeria) */
+		{ "ar-MA", 0x1801 }, /* Arabic (Morocco) */
+		{ "ar-TN", 0x1c01 }, /* Arabic (Tunisia) */
+		{ "ar-OM", 0x2001 }, /* Arabic (Oman) */
+		{ "ar-YE", 0x2401 }, /* Arabic (Yemen) */		
+		{ "ar-SY", 0x2801 }, /* Arabic (Syria) */
+		{ "ar-JO", 0x2c01 }, /* Arabic (Jordan) */
+		{ "ar-LB", 0x3001 }, /* Arabic (Lebanon) */
+		{ "ar-KW", 0x3401 }, /* Arabic (Kuwait) */
+		{ "ar-AE", 0x3801 }, /* Arabic (United Arab Emirates) */
+		{ "ar-BH", 0x3c01 }, /* Arabic (Bahrain) */		
+		{ "ar-QA", 0x4001 }, /* Arabic (Qatar) */
+		{ "as", 0x044d }, /* Assamese */
+		{ "az", 0x042c }, /* Azerbaijani */
+		{ "hy-AM", 0x042b }, /* Armenian */
+		{ "az", 0x044c }, /* Azeri (Latin) az- */
+		{ "az", 0x082c }, /* Azeri (Cyrillic) az- */
+		{ "eu-ES", 0x042d }, /* Basque */
+		{ "be-BY", 0x0423 }, /* Belarussian */		
+		{ "bn", 0x0445 }, /* Bengali bn- */
+		{ "bg-BG", 0x0402 }, /* Bulgarian */
+		{ "ca-ES", 0x0403 }, /* Catalan */
+		{ "zh-TW", 0x0404 }, /* Chinese (Taiwan) */
+		{ "zh-CN", 0x0804 }, /* Chinese (PRC) */
+		{ "zh-HK", 0x0c04 }, /* Chinese (Hong Kong) */		
+		{ "zh-SG", 0x1004 }, /* Chinese (Singapore) */
+		{ "ch-MO", 0x1404 }, /* Chinese (Macau SAR) */
+		{ "hr-HR", 0x041a }, /* Croatian */
+		{ "cs-CZ", 0x0405 }, /* Czech */
+		{ "da-DK", 0x0406 }, /* Danish */
+		{ "div", 0x465 }, /* Divehi div-*/
+		{ "nl-NL", 0x0413 }, /* Dutch (Netherlands) */		
+		{ "nl-BE", 0x0813 }, /* Dutch (Belgium) */
+		{ "en-US", 0x0409 }, /* English (USA) */
+		{ "en-GB", 0x0809 }, /* English (UK) */
+		{ "en-AU", 0x0c09 }, /* English (Australia) */
+		{ "en-CA", 0x1009 }, /* English (Canada) */
+		{ "en-NZ", 0x1409 }, /* English (New Zealand) */		
+		{ "en-IE", 0x1809 }, /* English (Ireland) */
+		{ "en-ZA", 0x1c09 }, /* English (South Africa) */
+		{ "en-JM", 0x2009 }, /* English (Jamaica) */
+		{ "en", 0x2409 }, /* English (Caribbean) */
+		{ "en-BZ", 0x2809 }, /* English (Belize) */
+		{ "en-TT", 0x2c09 }, /* English (Trinidad) */		
+		{ "en-ZW", 0x3009 }, /* English (Zimbabwe) */
+		{ "en-PH", 0x3409 }, /* English (Phillipines) */
+		{ "et-EE", 0x0425 }, /* Estonian */
+		{ "fo", 0x0438 }, /* Faeroese fo- */
+		{ "fa-IR", 0x0429 }, /* Farsi */
+		{ "fi-FI", 0x040b }, /* Finnish */		
+		{ "fr-FR", 0x040c }, /* French (France) */
+		{ "fr-BE", 0x080c }, /* French (Belgium) */
+		{ "fr-CA", 0x0c0c }, /* French (Canada) */
+		{ "fr-CH", 0x100c }, /* French (Switzerland) */
+		{ "fr-LU", 0x140c }, /* French (Luxembourg) */
+		{ "fr-MC", 0x180c }, /* French (Monaco) */		
+		{ "gl", 0x0456 }, /* Galician gl- */
+		{ "ga-IE", 0x083c }, /* Irish Gaelic */
+		{ "gd-GB", 0x100c }, /* Scottish Gaelic */
+		{ "ka-GE", 0x0437 }, /* Georgian */
+		{ "de-DE", 0x0407 }, /* German (Germany) */
+		{ "de-CH", 0x0807 }, /* German (Switzerland) */
+		{ "de-AT", 0x0c07 }, /* German (Austria) */
+		{ "de-LU", 0x1007 }, /* German (Luxembourg) */
+		{ "de-LI", 0x1407 }, /* German (Liechtenstein) */		
+		{ "el-GR", 0x0408 }, /* Greek */
+		{ "gu", 0x0447 }, /* Gujarati gu- */
+		{ "ha", 0x0468 }, /* Hausa */
+		{ "he-IL", 0x040d }, /* Hebrew */
+		{ "hi-IN", 0x0439 }, /* Hindi */
+		{ "hu-HU", 0x040e }, /* Hungarian */
+		{ "is-IS", 0x040f }, /* Icelandic */		
+		{ "id-ID", 0x0421 }, /* Indonesian */
+		{ "iu", 0x045d }, /* Inkutitut */
+		{ "it-IT", 0x0410 }, /* Italian (Italy) */
+		{ "it-CH", 0x0810 }, /* Italian (Switzerland) */
+		{ "ja-JP", 0x0411}, /* Japanese */
+		{ "kn", 0x044b }, /* Kannada kn- */
+		{ "ks", 0x0860 }, /* Kashmiri (India) ks- */		
+		{ "kk", 0x043f }, /* Kazakh kk- */
+		{ "kok", 0x0457 }, /* Konkani kok- */
+		{ "ko-KR", 0x0412 }, /* Korean */
+		{ "ko", 0x0812 }, /* Korean (Johab) ko- */
+		{ "kir", 0x0440 }, /* Kyrgyz */
+		{ "la", 0x0476 }, /* Latin */
+		{ "lo", 0x0454 }, /* Laothian */
+		{ "lv-LV", 0x0426 }, /* Latvian */
+		{ "lt-LT", 0x0427 }, /* Lithuanian */		
+		{ "lt-LT", 0x0827 }, /* Lithuanian (Classic) */
+		{ "mk", 0x042f }, /* FYRO Macedonian */
+		{ "my-MY", 0x043e }, /* Malaysian */
+		{ "my-BN", 0x083e }, /* Malay Brunei Darussalam */
+		{ "ml", 0x044c }, /* Malayalam ml- */
+		{ "mr", 0x044e }, /* Marathi mr- */
+		{ "mt", 0x043a }, /* Maltese */
+		{ "mo", 0x0450 }, /* Mongolian */
+		{ "ne-NP", 0x0461 }, /* Napali (Nepal) */
+		{ "ne-IN", 0x0861 }, /* Nepali (India) */
+		{ "nb-NO", 0x0414 }, /* Norwegian (Bokmai) */
+		{ "nn-NO", 0x0814 }, /* Norwegian (Nynorsk) */
+		{ "or", 0x0448 }, /* Oriya or- */
+		{ "om", 0x0472 }, /* Oromo (Afan, Galla) */
+		{ "pl-PL", 0x0415 }, /* Polish */		
+		{ "pt-BR", 0x0416 }, /* Portuguese (Brazil) */
+		{ "pt-PT", 0x0816 }, /* Portuguese (Portugal) */
+		{ "pa", 0x0446 }, /* Punjabi pa- */
+		{ "ps", 0x0463 }, /* Pashto (Pushto) */
+		{ "rm", 0x0417 }, /* Rhaeto-Romanic rm- */
+		{ "ro-RO", 0x0418 }, /* Romanian */
+		{ "ro-MD", 0x0818 }, /* Romanian (Moldova) */		
+		{ "ru-RU", 0x0419 }, /* Russian */
+		{ "ru-MD", 0x0819 }, /* Russian (Moldova) */
+		{ "se", 0x043b }, /* Sami (Lappish) se- */
+		{ "sa", 0x044f }, /* Sanskrit sa- */
+		{ "sr", 0x0c1a }, /* Serbian (Cyrillic) sr- */
+		{ "sr", 0x081a }, /* Serbian (Latin) sr- */		
+		{ "sd", 0x0459 }, /* Sindhi sd- */
+		{ "sk-SK", 0x041b }, /* Slovak */
+		{ "sl-SI", 0x0424 }, /* Slovenian */
+		{ "wen", 0x042e }, /* Sorbian wen- */
+		{ "so", 0x0477 }, /* Somali */
+		{ "es-ES", 0x040a }, /* Spanish (Spain, Traditional) */
+		{ "es-MX", 0x080a }, /* Spanish (Mexico) */		
+		{ "es-ES", 0x0c0a }, /* Spanish (Modern) */
+		{ "es-GT", 0x100a }, /* Spanish (Guatemala) */
+		{ "es-CR", 0x140a }, /* Spanish (Costa Rica) */
+		{ "es-PA", 0x180a }, /* Spanish (Panama) */
+		{ "es-DO", 0x1c0a }, /* Spanish (Dominican Republic) */
+		{ "es-VE", 0x200a }, /* Spanish (Venezuela) */		
+		{ "es-CO", 0x240a }, /* Spanish (Colombia) */
+		{ "es-PE", 0x280a }, /* Spanish (Peru) */
+		{ "es-AR", 0x2c0a }, /* Spanish (Argentina) */
+		{ "es-EC", 0x300a }, /* Spanish (Ecuador) */
+		{ "es-CL", 0x340a }, /* Spanish (Chile) */
+		{ "es-UY", 0x380a }, /* Spanish (Uruguay) */		
+		{ "es-PY", 0x3c0a }, /* Spanish (Paraguay) */
+		{ "es-BO", 0x400a }, /* Spanish (Bolivia) */
+		{ "es-SV", 0x440a }, /* Spanish (El Salvador) */
+		{ "es-HN", 0x480a }, /* Spanish (Honduras) */
+		{ "es-NI", 0x4c0a }, /* Spanish (Nicaragua) */
+		{ "es-PR", 0x500a }, /* Spanish (Puerto Rico) */		
+		{ "sx", 0x0430 }, /* Sutu */
+		{ "sw", 0x0441 }, /* Swahili (Kiswahili/Kenya) */
+		{ "sv-SE", 0x041d }, /* Swedish */
+		{ "sv-FI", 0x081d }, /* Swedish (Finland) */
+		{ "ta", 0x0449 }, /* Tamil ta- */
+		{ "tt", 0x0444 }, /* Tatar (Tatarstan) tt- */		
+		{ "te", 0x044a }, /* Telugu te- */
+		{ "th-TH", 0x041e }, /* Thai */
+		{ "ts", 0x0431 }, /* Tsonga ts- */
+		{ "tn", 0x0432 }, /* Tswana tn- */
+		{ "tr-TR", 0x041f }, /* Turkish */
+		{ "tl", 0x0464 }, /* Tagalog */
+		{ "tg", 0x0428 }, /* Tajik */
+		{ "bo", 0x0451 }, /* Tibetan */
+		{ "ti", 0x0473 }, /* Tigrinya */
+		{ "uk-UA", 0x0422 }, /* Ukrainian */		
+		{ "ur-PK", 0x0420 }, /* Urdu (Pakistan) */
+		{ "ur-IN", 0x0820 }, /* Urdu (India) */
+		{ "uz", 0x0443 }, /* Uzbek (Latin) uz- */
+		{ "uz", 0x0843 }, /* Uzbek (Cyrillic) uz- */
+		{ "ven", 0x0433 }, /* Venda ven- */
+		{ "vi-VN", 0x042a }, /* Vietnamese */
+		{ "cy-GB", 0x0452 }, /* Welsh */
+		{ "xh", 0x0434 }, /* Xhosa xh */
+		{ "yi", 0x043d }, /* Yiddish yi- */
+		{ "yo", 0x046a }, /* Yoruba */
+		{ "zu", 0x0435 }, /* Zulu zu- */
+		{ "en-US", 0x0800 } /* Default */
+	};
+
+#define NrMappings (sizeof(mLanguageIds)/sizeof(mLanguageIds[0]))
 
 U16 wvLangToLIDConverter ( const char * lang )
 {
-  if ( !lang || !strcmp(lang, "en-US") )
-    return 0x0409;
-  
-  if (!strcmp(lang, "af-ZA"))
-    return 0x0436;
-  else if(!strcmp(lang, "af-EG"))
-    return 0x0c01;
-  else if(!strcmp(lang, "af-SA"))
-    return 0x0401;
-  else if(!strcmp(lang, "zh-HK"))
-    return 0x0c04;
-  else if(!strcmp(lang, "zh-CN"))
-    return 0x0804;
-  else if(!strcmp(lang, "zh-SG"))
-    return 0x1004;
-  else if(!strcmp(lang, "zh-TW"))
-    return 0x0404;
-  else if(!strcmp(lang, "cs-CZ"))
-    return 0x0405;
-  else if(!strcmp(lang, "da-DK"))
-    return 0x0406;
-  else if(!strcmp(lang, "de-AT"))
-    return 0x0c07;
-  else if(!strcmp(lang, "de-DE"))
-    return 0x0407;
-  else if(!strcmp(lang, "el-GR"))
-    return 0x0408;
-  else if(!strcmp(lang, "en-AU"))
-    return 0x0c09;
-  else if(!strcmp(lang, "en-CA"))
-    return 0x1009;
-  else if(!strcmp(lang, "en-GB"))
-    return 0x0809;
-  else if(!strcmp(lang, "en-IE"))
-    return 0x1809;
-  else if(!strcmp(lang, "en-NZ"))
-    return 0x1409;
-  else if(!strcmp(lang, "es-ES"))
-    return 0x040a;
-  else if(!strcmp(lang, "es-MX"))
-    return 0x080a;
-  else if(!strcmp(lang, "fa-IR"))
-    return 0x040b;
-  else if(!strcmp(lang, "fr-FR"))
-    return 0x040c;
-  else if(!strcmp(lang, "hi-IN"))
-    return 0x0439;
-  else if(!strcmp(lang, "hy-AM"))
-    return 0x042b;
-  else if(!strcmp(lang, "it-IT"))
-    return 0x0410;
-  else if(!strcmp(lang, "iw-IL"))
-    return 0x040d;
-  else if(!strcmp(lang, "ka-GE"))
-    return 0x0437;
-  else if(!strcmp(lang, "ja-JP"))
-    return 0x0411;
-  else if(!strcmp(lang, "ko-KR"))
-    return 0x0412;
-  else if(!strcmp(lang, "nl-NL"))
-    return 0x0413;
-  else if(!strcmp(lang, "nb-NO"))
-    return 0x0414;
-  else if(!strcmp(lang, "nn-NO"))
-    return 0x0814;
-  else if(!strcmp(lang, "pt-BR"))
-    return 0x0416;
-  else if(!strcmp(lang, "pt-PT"))
-    return 0x0816;
-  else if(!strcmp(lang, "ru-RU"))
-    return 0x0419;
-  else if(!strcmp(lang, "sv-SE"))
-    return 0x041d;
-  else if(!strcmp(lang, "th-TH"))
-    return 0x041e;
-  else if(!strcmp(lang, "tr-TR"))
-    return 0x041f;
-  else if(!strcmp(lang, "vi-VN"))
-    return 0x042a;
-  else if(!strcmp(lang, "lt-LT"))
-    return 0x0427;
-  else if(!strcmp(lang, "-none-"))
-    return 0x0400;
-  else
-    return 0x0409;
+	unsigned int i = 0 ;
+
+	if (!lang)
+		return 0x0400;   /* return -none- */
+	
+	for ( i = 0 ; i < NrMappings ; i++ )
+		if (!strcmp (lang, mLanguageIds[i].language_tag))
+			return mLanguageIds[i].lid ;
+	
+	return 0x0400 ;   /* return -none- */
 }
 
-char *
+const char *
 wvLIDToLangConverter (U16 lid)
 {
-  switch (lid)
-    {
-#if 0
-    case 0x0000: /* Language Neutral */
-#endif
+	unsigned int i = 0 ;
+	
+	if ( lid == 0 ) /* language netural */
+		return "-none-" ;
+	
+	for ( i = 0 ; i < NrMappings ; i++ )
+		if ( mLanguageIds[i].lid == lid )
+			return mLanguageIds[i].language_tag ;
 
-    case 0x0436: 
-      return "af-ZA";
-
-#if 0
-    case 0x041c: /* Albanian */
-
-    case 0x1401: /* Arabic (Algeria) */
-    case 0x3c01: /* Arabic (Bahrain) */
-#endif
-    case 0x0c01: /* Arabic (Egypt) */
-      return "af-EG";
-#if 0
-    case 0x0801: /* Arabic (Iraq) */
-    case 0x2c01: /* Arabic (Jordan) */
-    case 0x3401: /* Arabic (Kuwait) */
-    case 0x3001: /* Arabic (Lebanon) */
-    case 0x1001: /* Arabic (Libya) */
-    case 0x1801: /* Arabic (Morocco) */
-    case 0x2001: /* Arabic (Oman) */
-    case 0x4001: /* Arabic (Qatar) */
-#endif
-    case 0x0401: /* Arabic (Saudi Arabia) */
-      return "af-SA";
-#if 0
-    case 0x2801: /* Arabic (Syria) */
-    case 0x1c01: /* Arabic (Tunisia) */
-    case 0x3801: /* Arabic (U.A.E.) */
-    case 0x2401: /* Arabic (Yemen) */
-#endif
-
-#if 0
-    case 0x044d: /* Windows 2000: Assamese. This is Unicode only. */
-    case 0x082c: /* Azeri (Cyrillic) */
-    case 0x042c: /* Azeri (Latin) */
-    case 0x042d: /* Basque */
-    case 0x0423: /* Belarussian */
-    case 0x0445: /* Windows 2000: Bengali. This is Unicode only. */
-    case 0x0402: /* Bulgarian */
-    case 0x0455: /* Burmese */
-    case 0x0403: /* Catalan */
-#endif
-
-    case 0x0c04: /* Chinese (Hong Kong SAR, PRC) */
-      return "zh-HK";
-#if 0
-    case 0x1404: /* Chinese (Macau SAR) */
-#endif
-    case 0x0804: /* Chinese (PRC) */
-      return "zh-CN";
-    case 0x1004: /* Chinese (Singapore) */
-      return "zh-SG";
-    case 0x0404: /* Chinese (Taiwan) */
-      return "zh-TW";
-
-#if 0
-    case 0x041a: /* Croatian */
-#endif
-
-	case 0x0405: 
-      return "cs-CZ";
-
-    case 0x0406:  
-      return "da-DK";
-
-    case 0x0c07: /* German (Austria) */
-      return "de-AT";
-    case 0x0407: /* german */
-      return "de-DE";
-#if 0
-    case 0x1407: /* German (Liechtenstein) */
-    case 0x1007: /* German (Luxembourg) */
-    case 0x0807: /* swiss german */
-#endif
-
-    case 0x0408: /* Greek */
-      return "el-GR";
-
-    case 0x0c09: /* English (Australian) */
-      return "en-AU";
-#if 0
-    case 0x2809: /* English (Belize) */
-#endif
-    case 0x1009: /* English (Canadian) */
-      return "en-CA";
-#if 0
-    case 0x2409: /* English (Caribbean) */
-#endif
-    case 0x0809: /* british english */
-      return "en-GB";
-    case 0x1809: /* English (Ireland) */
-      return "en-IE";
-#if 0
-    case 0x2009: /* English (Jamaica) */
-#endif
-    case 0x1409: /* English (New Zealand) */
-      return "en-NZ";
-#if 0
-    case 0x3409: /* English (Philippines) */
-    case 0x2c09: /* English (Trinidad) */
-#endif
-    case 0x1c09: /* English (South Africa) */
-      return "en-ZA";
-#if 0
-    case 0x3009: /* English (Zimbabwe) */
-#endif
-
-#if 0
-    case 0x2c0a: /* Spanish (Argentina) */
-    case 0x400a: /* Spanish (Bolivia) */
-    case 0x340a: /* Spanish (Chile) */
-    case 0x240a: /* Spanish (Colombia) */
-    case 0x140a: /* Spanish (Costa Rica) */
-    case 0x1c0a: /* Spanish (Dominican Republic) */
-    case 0x300a: /* Spanish (Ecuador) */
-    case 0x440a: /* Spanish (El Salvador) */
-#endif
-    case 0x040a: /* castillian - traditional sort */
-      return "es-ES";
-#if 0
-    case 0x100a: /* Spanish (Guatemala) */
-    case 0x480a: /* Spanish (Honduras) */
-#endif
-    case 0x080a: /* mexican */ 
-      return "es-MX";
-#if 0
-    case 0x0c0a: /* Spanish (Modern Sort) */
-    case 0x4c0a: /* Spanish (Nicaragua) */
-    case 0x180a: /* Spanish (Panama) */
-    case 0x3c0a: /* Spanish (Paraguay) */
-    case 0x280a: /* Spanish (Peru) */
-    case 0x500a: /* Spanish (Puerto Rico) */
-    case 0x380a: /* Spanish (Uruguay) */
-    case 0x200a: /* Spanish (Venezuela) */
-#endif
-
-#if 0
-    case 0x0425: /* Estonian */
-    case 0x0438: /* Faeroese */
-#endif
-
-    case 0x0429: /* Farsi */
-      return "fa-IR";
-
-	case 0x040b:
-      return "fi-FI";
-
-#if 0
-    case 0x080c: /* French (Belgian) */
-    case 0x0c0c: /* French (Canadian) */
-#endif
-	case 0x040c:
-      return "fr-FR";
-#if 0
-    case 0x140c: /* French (Luxembourg) */
-    case 0x180c: /* French (Monaco) */
-    case 0x100c: /* French (Switzerland) */
-#endif
-
-#if 0
-    case 0x0447: /* Windows 2000: Gujarati. This is Unicode only. */
-#endif
-
-    case 0x0439: /* Windows 2000: Hindi. This is Unicode only. */
-      return "hi-IN";
-
-    case 0x042b: /* Windows 2000: Armenian. This is Unicode only. */
-      return "hy-AM";
-
-    case 0x0410:
-      return "it-IT";
-
-    case 0x040d: /* hebrew */
-      return "iw-IL";
-
-#if 0
-    case 0x040e: /* Hungarian */
-#endif
-
-    case 0x0437: /* Windows 2000: Georgian. This is Unicode only. */
-      return "ka-GE";
-
-#if 0
-    case 0x040f: /* Icelandic */
-    case 0x0421: /* Indonesian */
-    case 0x0410: /* Italian (Standard) */
-    case 0x0810: /* Italian (Switzerland) */
-#endif
-
-    case 0x0411: /* Japanese */
-      return "ja-JP";
-
-#if 0
-    case 0x044b: /* Windows 2000: Kannada. This is Unicode only. */
-    case 0x0860: /* Kashmiri (India) */
-    case 0x043f: /* Kazakh */
-    case 0x0457: /* Windows 2000: Konkani. This is Unicode only. */
-#endif
-
-#if 0
-    case 0x0812: /* Korean (Johab) */
-#endif
-    case 0x0412: /* Korean */
-      return "ko-KR";
-
-    case 0x0827: /* Lithuanian (Classic) */
-    case 0x0427: /* Lithuanian */
-      return "lt-LT";
-
-#if 0
-    case 0x0426: /* Latvian */
-    case 0x042f: /* Macedonian */
-    case 0x083e: /* Malay (Brunei Darussalam) */
-    case 0x043e: /* Malay (Malaysian) */
-    case 0x044c: /* Windows 2000: Malayalam. This is Unicode only. */
-    case 0x0458: /* Manipuri */
-    case 0x044e: /* Windows 2000: Marathi. This is Unicode only. */
-#endif
-
-#if 0
-    case 0x0813: /* Dutch (Belgium) */
-#endif
-    case 0x0413: /* Dutch (Netherlands) */
-      return "nl-NL";
-
-#if 0
-    case 0x0861: /* Nepali Windows 2000: (India). This is Unicode only. */
-#endif
-
-    case 0x0414: /* Norwegian (Bokmal) */
-      return "nb-NO";
-    case 0x0814: /* Norwegian (Nynorsk) */
-      return "nn-NO";
-
-#if 0
-    case 0x0448: /* Windows 2000: Oriya. This is Unicode only. */
-    case 0x0415: /* Polish */
-#endif
-
-    case 0x0416: /* brazilian */
-      return "pt-BR";
-    case 0x0816: /* portugese */
-      return "pt-PT";
-
-#if 0
-    case 0x0446: /* Windows 2000: Punjabi. This is Unicode only. */
-    case 0x0418: /* Romanian */
-#endif
-
-    case 0x0419:
-      return "ru-RU";
-
-#if 0
-    case 0x044f: /* Windows 2000: Sanskrit. This is Unicode only. */
-    case 0x0c1a: /* Serbian (Cyrillic) */
-    case 0x081a: /* Serbian (Latin) */
-    case 0x0459: /* Sindhi */
-    case 0x041b: /* Slovak */
-    case 0x0424: /* Slovenian */
-    case 0x0430: /* Sutu */
-    case 0x0441: /* Swahili (Kenya) */
-#endif
-
-#if 0
-    case 0x081d: /* Swedish (Finland) */
-#endif
-    case 0x041d:
-      return "sv-SE";
-
-#if 0
-    case 0x0449: /* Windows 2000: Tamil. This is Unicode only. */
-    case 0x0444: /* Tatar (Tatarstan) */
-    case 0x044a: /* Windows 2000: Telugu. This is Unicode only. */
-#endif
-
-    case 0x041e: /* Thai */
-      return "th-TH";
-
-    case 0x041f: /* Turkish */
-      return "tr-TR";
-
-#if 0
-    case 0x0422: /* Ukrainian */
-    case 0x0820: /* Urdu (India) */
-    case 0x0420: /* Urdu (Pakistan) */
-    case 0x0843: /* Uzbek (Cyrillic) */
-    case 0x0443: /* Uzbek (Latin) */
-#endif
-
-    case 0x042a: /* Vietnamese */
-      return "vi-VN";
-
-    case 0x0400:
-      return "-none-";
-
-    case 0x0409:
-    default:
-      return "en-US";
-    }
+	return "-none-"; /* default */
 }
 
-char *
+const char *
 wvLIDToCodePageConverter (U16 lid)
 {
     if (lid == 0x0FFF)	/*Macintosh Hack */
@@ -647,17 +455,16 @@ wvLIDToCodePageConverter (U16 lid)
       case 0x16:		/*Portuguese */
 	  return ("CP1252");
       case 0x17:		/*Rhaeto-Romanic */
-	  return ("CP1252");	/* ? */
+	  return ("CP1252");
       case 0x18:		/*Romanian */
 	  return ("CP1250");
       case 0x19:		/*Russian */
 	  return ("CP1251");
-      case 0x1a:		/*Serbian, Croatian */
+      case 0x1a:		/*Serbian, Croatian, (Bosnian?) */
       switch (lid)
 		{
-#if 0
       	case 0x041a:		/*Croatian */
-#endif
+	  	return ("CP1252");
       	case 0x0c1a:		/*Serbian (Cyrillic) */
 	  	return ("CP1251");
       	case 0x081a:		/*Serbian (Latin) */
@@ -673,13 +480,13 @@ wvLIDToCodePageConverter (U16 lid)
 	  return ("CP874");
       case 0x1f:		/*Turkish */
 	  return ("CP1254");
-      case 0x20:		/*Urdu */
-	  return ("CP1256");
-      case 0x21:		/*Bahasa / Indonesian */
-	  return ("CP1256");
+      case 0x20:		/*Urdu. This is Unicode only. */
+	  return ("0");
+      case 0x21:		/*Bahasa Indonesian */
+	  return ("CP1252");
       case 0x22:		/*Ukrainian */
 	  return ("CP1251");
-      case 0x23:		/*Byelorussian / Belarussian */
+      case 0x23:		/*Byelorussian / Belarusian */
 	  return ("CP1251");
       case 0x24:		/*Slovenian */
 	  return ("CP1250");
@@ -689,8 +496,8 @@ wvLIDToCodePageConverter (U16 lid)
 	  return ("CP1257");
       case 0x27:		/*Lithuanian */
 	  return ("CP1257");
-      case 0x29:		/*Farsi */
-	  return ("CP1256");
+      case 0x29:		/*Farsi / Persian. This is Unicode only. */
+	  return ("0");
       case 0x2a:		/*Vietnamese */
 	  return ("CP1258");
       case 0x2b:		/*Windows 2000: Armenian. This is Unicode only. */
@@ -715,17 +522,17 @@ wvLIDToCodePageConverter (U16 lid)
 	  return ("CP1252");
       case 0x37:		/*Windows 2000: Georgian. This is Unicode only. */
 	  return ("CP0");
-#if 0
       case 0x38:		/*Faeroese */
-#endif
+	  return ("CP1252");
       case 0x39:		/*Windows 2000: Hindi. This is Unicode only. */
 	  return ("CP0");
       case 0x3E:		/*Malaysian / Malay */
-	  return ("CP1251");
+	  return ("CP1252");
 #if 0
       case 0x3f:		/*Kazakh */
-      case 0x41:		/*Swahili */
 #endif
+      case 0x41:		/*Swahili */
+	  return ("CP1252");
       case 0x43:		/*Uzbek */
       switch (lid)
 		{
@@ -750,9 +557,8 @@ wvLIDToCodePageConverter (U16 lid)
       case 0x4e:		/*Windows 2000: Marathi. This is Unicode only. */
       case 0x4f:		/*Windows 2000: Sanskrit. This is Unicode only. */
 	  return ("CP0");
-#if 0
-      case 0x55:		/*Burmese */
-#endif
+      case 0x55:		/*Myanmar / Burmese. This is Unicode only. */
+	  return ("CP0");
       case 0x57:		/*Windows 2000: Konkani. This is Unicode only. */
 	  return ("CP0");
 #if 0
@@ -764,6 +570,7 @@ wvLIDToCodePageConverter (U16 lid)
 	  return ("CP0");
       };
 
+	/* TODO output a warning since this is a guess */
     return ("CP1252");
 }
 
@@ -773,7 +580,7 @@ swap_iconv (U16 lid)
     iconv_t handle = NULL;
     char f_code[33];		/* From CCSID                           */
     char t_code[33];		/* To CCSID                             */
-    char *codepage = NULL;
+    const char *codepage = NULL;
     size_t ibuflen, obuflen;
 
     U8 buffer[2];
@@ -802,6 +609,8 @@ swap_iconv (U16 lid)
     strcpy (t_code, "UCS-2");
 
     handle = iconv_open (t_code, f_code);
+    if (handle == (iconv_t)-1)
+            return 0;
 
     buffer[0] = 0x20 & 0xff;
     buffer[1] = 0;
@@ -821,7 +630,7 @@ wvHandleCodePage (U16 eachchar, U16 lid)
 {
     char f_code[33];		/* From CCSID                           */
     char t_code[33];		/* To CCSID                             */
-    char *codepage;
+    const char *codepage;
     iconv_t iconv_handle;	/* Conversion Descriptor returned       */
     /* from iconv_open() function           */
     size_t ibuflen;		/* Length of input buffer               */
@@ -907,7 +716,7 @@ wvOutputFromUnicode (U16 eachchar, char *outputtype)
 
     if ((iconv_handle == (iconv_t)-1) || strcmp (cached_outputtype, outputtype) != 0)
       {
-	  if (iconv_handle)
+	  if ((iconv_handle != (iconv_t)-1))
 	      iconv_close (iconv_handle);
 
 	  iconv_handle = iconv_open (outputtype, "UCS-2");
@@ -976,49 +785,6 @@ wvOutputFromUnicode (U16 eachchar, char *outputtype)
       }
 }
 
-
-void
-wvBeginDocument (expand_data * data)
-{
-    if ((data->sd != NULL) && (data->sd->elements[TT_DOCUMENT].str[0] != NULL))
-      {
-	  wvTrace (("doc begin is %s", data->sd->elements[TT_DOCUMENT].str[0]));
-	  wvExpand (data, data->sd->elements[TT_DOCUMENT].str[0],
-		    strlen (data->sd->elements[TT_DOCUMENT].str[0]));
-	  if (data->retstring)
-	    {
-		wvTrace (("doc begin is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-void
-wvEndDocument (expand_data * data)
-{
-    PAP apap;
-    /*
-       just for html mode, as this is designed for, I always have an empty
-       para end just to close off any open lists
-     */
-    wvInitPAP (&apap);
-    data->props = (void *) &apap;
-    wvEndPara (data);
-
-    if ((data->sd != NULL) && (data->sd->elements[TT_DOCUMENT].str[1] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_DOCUMENT].str[1],
-		    strlen (data->sd->elements[TT_DOCUMENT].str[1]));
-	  if (data->retstring)
-	    {
-		wvTrace (("doc end is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
 int
 wvHandleElement (wvParseStruct * ps, wvTag tag, void *props, int dirty)
 {
@@ -1037,226 +803,6 @@ wvHandleDocument (wvParseStruct * ps, wvTag tag)
     return (0);
 }
 
-void
-wvBeginSection (expand_data * data)
-{
-    if (data != NULL)
-	data->asep = (SEP *) data->props;
-
-    if ((data != NULL) && (data->sd != NULL)
-	&& (data->sd->elements[TT_SECTION].str != NULL)
-	&& (data->sd->elements[TT_SECTION].str[0] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_SECTION].str[0],
-		    strlen (data->sd->elements[TT_SECTION].str[0]));
-	  if (data->retstring)
-	    {
-		wvTrace (("para begin is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-
-int
-wvIsEmptyPara (PAP * apap, expand_data * data, int inc)
-{
-    /* 
-       if we are a end of table para then i consist of nothing that is of
-       any use for beginning of a para
-     */
-    if (apap == NULL)
-	return (0);
-
-    if (apap->fTtp == 1)
-	return (1);
-
-    /* 
-       if i consist of a vertically merged cell that is not the top one, then
-       also i am of no use
-     */
-    if (apap->fInTable == 1)
-      {
-#if 0
-	  wvTrace (
-		   ("This Para is in cell %d %d\n", data->whichrow,
-		    data->whichcell));
-	  if (*data->vmerges)
-	    {
-		/* only ignore a vertically merged cell if the setting in the config file have been set that way */
-		if (data
-		    && data->sd
-		    && data->sd->elements[TT_TABLEOVERRIDES].str
-		    && data->sd->elements[TT_TABLEOVERRIDES].str[5])
-		  {
-		      if ((*data->vmerges)[data->whichrow][data->whichcell] ==
-			  0)
-
-			{
-			    wvTrace (("Skipping the next paragraph\n"));
-			    if (inc)
-				data->whichcell++;
-			    return (1);
-			}
-		  }
-	    }
-#else
-	  return 0;
-#endif
-      }
-    return (0);
-}
-
-void
-wvBeginComment (expand_data * data)
-{
-    if (data != NULL)
-      {
-	  wvTrace (("comment beginning\n"));
-	  if ((data->sd != NULL) && (data->sd->elements[TT_COMMENT].str)
-	      && (data->sd->elements[TT_COMMENT].str[0] != NULL))
-	    {
-		wvExpand (data, data->sd->elements[TT_COMMENT].str[0],
-			  strlen (data->sd->elements[TT_COMMENT].str[0]));
-		if (data->retstring)
-		  {
-		      printf ("%s", data->retstring);
-		      wvFree (data->retstring);
-		  }
-	    }
-      }
-}
-
-void
-wvEndComment (expand_data * data)
-{
-    if ((data->sd != NULL) && (data->sd->elements[TT_COMMENT].str)
-	&& (data->sd->elements[TT_COMMENT].str[1] != NULL))
-      {
-	  wvTrace (("comment ending\n"));
-	  wvExpand (data, data->sd->elements[TT_COMMENT].str[1],
-		    strlen (data->sd->elements[TT_COMMENT].str[1]));
-	  if (data->retstring)
-	    {
-		wvTrace (("comment end is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-void
-wvBeginPara (expand_data * data)
-{
-    if (wvIsEmptyPara ((PAP *) data->props, data, 1))
-	return;
-
-    if (data != NULL)
-      {
-	  wvTrace (
-		   ("para of style %d beginning\n",
-		    ((PAP *) (data->props))->istd));
-	  if ((data->sd != NULL) && (data->sd->elements[TT_PARA].str)
-	      && (data->sd->elements[TT_PARA].str[0] != NULL))
-	    {
-		wvExpand (data, data->sd->elements[TT_PARA].str[0],
-			  strlen (data->sd->elements[TT_PARA].str[0]));
-		if (data->retstring)
-		  {
-		      printf ("%s", data->retstring);
-		      wvFree (data->retstring);
-		  }
-	    }
-      }
-    wvTrace (
-	     ("This Para is out cell %d %d \n", data->whichrow,
-	      data->whichcell));
-}
-
-void
-wvEndPara (expand_data * data)
-{
-    if ((data->sd != NULL) && (data->sd->elements[TT_PARA].str)
-	&& (data->sd->elements[TT_PARA].str[1] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_PARA].str[1],
-		    strlen (data->sd->elements[TT_PARA].str[1]));
-	  if (data->retstring)
-	    {
-		wvTrace (("para end is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-void
-wvEndSection (expand_data * data)
-{
-    if ((data != NULL) && (data->sd != NULL)
-	&& (data->sd->elements[TT_SECTION].str != NULL)
-	&& (data->sd->elements[TT_SECTION].str[1] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_SECTION].str[1],
-		    strlen (data->sd->elements[TT_SECTION].str[1]));
-	  if (data->retstring)
-	    {
-		wvTrace (("para end is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-void
-wvBeginCharProp (expand_data * data, PAP * apap)
-{
-    CHP *achp;
-
-    if (wvIsEmptyPara (apap, data, 0))
-	return;
-
-    achp = (CHP *) data->props;
-    wvTrace (("beginning character run\n"));
-    if (achp->ico)
-      {
-	  wvTrace (("color is %d\n", achp->ico));
-      }
-
-    if ((data != NULL) && (data->sd != NULL)
-	&& (data->sd->elements[TT_CHAR].str)
-	&& (data->sd->elements[TT_CHAR].str[0] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_CHAR].str[0],
-		    strlen (data->sd->elements[TT_CHAR].str[0]));
-	  if (data->retstring)
-	    {
-		wvTrace (("char begin is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-void
-wvEndCharProp (expand_data * data)
-{
-    wvTrace (("ending character run\n"));
-    if ((data->sd != NULL) && (data->sd->elements[TT_CHAR].str)
-	&& (data->sd->elements[TT_CHAR].str[1] != NULL))
-      {
-	  wvExpand (data, data->sd->elements[TT_CHAR].str[1],
-		    strlen (data->sd->elements[TT_CHAR].str[1]));
-	  if (data->retstring)
-	    {
-		wvTrace (("char end is now %s", data->retstring));
-		printf ("%s", data->retstring);
-		wvFree (data->retstring);
-	    }
-      }
-}
-
 void
 wvSetCharHandler (wvParseStruct * ps,
 		  int (*proc) (wvParseStruct *, U16, U8, U16))
@@ -1284,30 +830,7 @@ wvSetDocumentHandler (wvParseStruct * ps,
 {
     ps->dochandler = proc;
 }
-
-void
-wvSetEntityConverter (expand_data * data)
-{
-    if ((data->sd) && (data->sd->elements[TT_CHARENTITY].str)
-	&& (data->sd->elements[TT_CHARENTITY].str[0]))
-      {
-	  wvExpand (data, data->sd->elements[TT_CHARENTITY].str[0],
-		    strlen (data->sd->elements[TT_CHARENTITY].str[0]));
-	  if (data->retstring)
-	    {
-		if (!(strcasecmp (data->retstring, "HTML")))
-		    wvConvertUnicodeToEntity = wvConvertUnicodeToHtml;
-		else if (!(strcasecmp (data->retstring, "LaTeX")))
-		    wvConvertUnicodeToEntity = wvConvertUnicodeToLaTeX;
-		wvTrace (
-			 ("Using %s entity conversion in conjunction with ordinary charset conversion\n",
-			  data->retstring));
-		wvFree (data->retstring);
-	    }
-      }
-}
-
-
+    
 int
 wvConvertUnicodeToLaTeX (U16 char16)
 {
@@ -2575,3 +2098,113 @@ thing and just put ... instead of this
     /* if (char16 >= 0x100) printf("[%x]", char16); */
     return (0);
 }
+
+
+
+int
+wvConvertUnicodeToXml (U16 char16)
+{
+    switch (char16)
+      {
+      case 11:
+	  printf ("<br />");
+	  return (1);
+
+      case 30:
+      case 31:
+      case 12:
+      case 13:
+      case 14:
+      case 7:
+	  return (1);
+
+      /* Much simpler here, because XML supports only a few entities */
+      case 34:
+	  printf ("&quot;");
+	  return (1);
+      case 38:
+	  printf ("&amp;");
+	  return (1);
+      case 39:
+	  printf ("&apos;");
+	  return (1);
+      case 60:
+	  printf ("&lt;");
+	  return (1);
+      case 62:
+	  printf ("&gt;");
+	  return (1);
+      }
+    
+    return (0);
+}
+
+char *str_copy(char *d, size_t n, char *s)
+{
+    strncpy(d, s, n);
+    d[n-1] = 0;
+    return d;
+}
+
+char *str_append(char *d, size_t n, char *s)
+{
+    int max = n - strlen(d) - 1;
+    strncat(d, s, max);
+    d[n-1] = 0;
+    return d;
+}    
+
+#define BUF_COPY(d,s) str_copy(d,sizeof(d),s)
+
+char *
+wvConvertStylename(char *stylename, char *outputtype)
+{
+    static char cached_outputtype[36];
+    static iconv_t iconv_handle = (iconv_t)-1;
+    /**FIXME: 100 is just the size of stylename[] from wv.h**/
+    static char buffer[100];
+    char *ibuf, *obuf;
+    size_t ibuflen, obuflen, len;
+
+    /* Destroy */
+    if(!outputtype) 
+    {
+	if ((iconv_handle != (iconv_t)-1))
+	    iconv_close(iconv_handle);
+	return NULL;
+    }
+
+    /* Initialize */
+    if(!iconv_handle || strcmp(cached_outputtype, outputtype))
+    {
+	if ((iconv_handle != (iconv_t)-1))
+	    iconv_close(iconv_handle);
+
+	/**FIXME: dont know if ISO-8859-1 is really the correct
+	 **charset for style names with eg umlauts.             **/
+	iconv_handle = iconv_open(outputtype, "ISO-8859-1");
+	if(iconv_handle == (iconv_t)-1)
+	{
+	    wvError(("iconv_open fail: %d, cannot convert %s to %s\n",
+		     errno, "ISO-8859-1", outputtype));
+	    return stylename;
+	}
+    
+	BUF_COPY(cached_outputtype, outputtype);
+    }
+
+    /* Convert */
+    ibuf    = stylename;
+    ibuflen = strlen(stylename);
+    obuf    = buffer;
+    obuflen = sizeof(buffer) - 1;
+    len     = wv_iconv (iconv_handle, &ibuf, &ibuflen, &obuf, &obuflen);
+    *obuf   = 0;
+    if(len == -1) 
+    {
+	wvError(("wvConfig.c: cant iconv()\n"));
+	return stylename;
+    }
+    
+    return buffer;
+}
