@@ -1291,7 +1291,6 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 	  wvTrace (("field middle\n"));
 	  if (achp->fOle2)
 	    {
-		/*pps_entry *test; */
 		wvError (
 			 ("this field has an associated embedded object of id %x\n",
 			  achp->fcPic_fcObj_lTagObj));
@@ -1335,7 +1334,7 @@ mySpecCharProc (wvParseStruct * ps, U16 eachchar, CHP * achp)
 			achp->fcPic_fcObj_lTagObj, achp->fObj, achp->fOle2));
 
 	      if (achp->fOle2)
-		  exit (139);
+		return (0);
 	      if(!no_graphics) 
 	      {
 	      wvStream_goto (ps->data, achp->fcPic_fcObj_lTagObj);
@@ -1592,7 +1591,7 @@ myCharProc (wvParseStruct * ps, U16 eachchar, U8 chartype, U16 lid)
 int
 wvOpenConfig (state_data *myhandle,char *config)
 {
-    char buf[BUFSIZ];
+    static char buf[BUFSIZ] = "";
     FILE *tmp;
     int i = 0;
     if (config == NULL)
